;****************************************
;
;	 "SAUCER" - A New Spectrum Game
;
;	Started On The 4th September 2006
;
;
;	(C) 2006 Frobush / Jonathan Smith
;
;****************************************

;****************************************
; LIBRARY INCLUDE FILE...
;****************************************

; ScreenOff:
; FillScreen:
; FillAttrBoxes:
; FillAttrBox:
; FillScreenBox:
; PrintShapeFX:
; PrintShapes:
; PrintShape:
; Pixad:
; DownLine:
; Random:
; PrintString:
; NibblePrinter:
; NibblePrint:

;****************************************
; INITIALISATION STUFF...
;****************************************

ScreenOff:
			xor		a
			ld		hl,0x00 * 0x20 + ATTRADDR + 0x00
			ld		bc,0x2018
			call	FillAttrBox			; switch the screen off!
			ex		af,af

FillScreen:
			ld		hl,SCRNADDR			; fill the screen with byte pattern
			ld		de,SCRNADDR+1
			ld		bc,0x1800 -1
			ld		(hl),a
			ldir
			ret

;****************************************
; ATTRIBUTE COLOUR BOX FILL...
;****************************************

; hl = table (attr, x size, ysize, address)

FillAttrBoxes:

			ld		a,(hl)				; attribute
			or		a
			ret		z
			ex		af,af
			inc		hl
			ld		c,(hl)				; y size
			inc		hl
			ld		b,(hl)				; x size
			inc		hl
			ld		a,(hl)
			inc		hl
			push	hl
			ld		h,(hl)
			ld		l,a					; attr screen address
			ex		af,af
			call	FillAttrBox
			pop		hl
			inc		hl
			jr		FillAttrBoxes

;****************************************

; bc = bx,cy chr box length  
; hl = attraddr  
; a = colour

FillAttrBox:

;		ld		a,WHITE * PAPER + WHITE
;		ld		hl,0x00 * 0x20 + ATTRADDR + 0x00
;		ld		bc,0x2018	; 32 * 24

			ex		af,af
			ld		a,32				; calculate downline offset (32 - x len)
			sub		b
			ld		e,a
			ld		d,0					; de = downline offset

			ld		a,b

AYRow:		ex		af,af

AXRow:		ld		(hl),a				; dump colour row / column
			inc		hl
			djnz	AXRow
			ex		af,af
			ld		b,a
			add		hl,de				; move down a line
			dec		c
			jr		nz,AYRow
			ret

;****************************************
; SCREEN BOX FILL...
;****************************************

; bc = bx,cy box length
; de = pixel coords
; a = fill byte

FillScreenBox:

			ex		af,af
			call	Pixad

			ld		a,b

BYRow:		ex		af,af
			push	hl
BXRow:		ld		(hl),a				; dump row / column
			inc		l
			djnz	BXRow
			pop		hl
			ld		b,a
			call	DownLine
			ld		a,b
			ex		af,af
			ld		b,a
			dec		c
			jr		nz,BYRow
			ret

;****************************************
; PRINT SHAPE ROUTINES...
;****************************************

; bc = bx,cy box length
; de = pixel coords
; hl = graphic address
;  a = mask byte

PrintShapeFX:
			ex		af,af
			push	hl
			call	Pixad				; hl = screen address
			pop		de					; de = graphic address
			ex		af,af

ShYRow:		push	bc
			push	hl
			ld		c,a
			ex		af,af

ShXRow:		ld		a,(de)
			and		c
			ld		(hl),a
			inc		de
			inc		l
			djnz	ShXRow

			pop		hl
			call	DownLine
			ex		af,af
			inc		a					; little fx
;			neg
;			rrca
			pop		bc
			dec		c
			jr		nz,ShYRow
			ret

;****************************************

; hl = data pointer

PrintShapes:
			ld		d,(hl)				; y or 0xff to terminate
			inc		d
			ret		z
			dec		d
			inc		hl
			ld		e,(hl)				; x
			inc		hl
			ld		c,(hl)				; yl
			inc		hl
			ld		b,(hl)				; xl
			inc		hl
			ld		a,(hl)
			inc		hl
			push	hl
			ld		h,(hl)
			ld		l,a					; graphic data
			call	PrintShape
			pop		hl
			inc		hl
			jr		PrintShapes

; bc = bx,cy box length
; de = dx,ey pixel coords
; hl = graphic address

PrintShape:	push	hl
			call	Pixad				; hl = screen address
			pop		de					; de = graphic address

ShYRow1:	push	hl
			ld		a,b
			ex		af,af
ShXRow1:	ld		a,(de)
			ld		(hl),a
			inc		de
			inc		l
			djnz	ShXRow1
			ex		af,af
			ld		b,a

			pop		hl
			call	DownLine

			dec		c
			jr		nz,ShYRow1
			ret

;****************************************
; GENERATE RANDOM NUMBERS...
;****************************************

Random:		ld		hl,Rand1
			ld		e,(hl)
			inc		l
			ld		d,(hl)
			inc		l

			ld		a,r
			xor		(hl)
			xor		e
			xor		d
			rlca
			rlca
			rlca
			srl		e
			srl		d
			ld		(hl),d
			dec		l
			ld		(hl),e
			dec		l
			ld		(hl),a
			ret

;****************************************
; SCREEN ADDRESSING ROUTINES...
;****************************************

; de = screen xy pixel positions (256 * 192)
; returns with hl as screen address

Pixad:		ld		a,d
			srl		a
			srl 	a
			srl 	a
			and 	0x18
			or 		0x40

			ld 		h,a
			ld 		a,d
			and 	7
			or 		h
			ld 		h,a

			ld 		a,d
			add 	a,a
			add 	a,a
			and 	0xe0
			ld 		l,a

			ld 		a,e
			srl 	a
			srl		a
			srl		a
			or		l
			ld		l,a					; hl = screen address.
			ret

;****************************************

; hl = screen address
; returns with hl as new screen address

DownLine:	inc 	h

DLine:		ld 		a,h
			and 	7
			ret 	nz
			ld 		a,l
			add 	a,32
			ld 		l,a
			ret 	c
			ld 		a,h
			sub 	8
			ld 		h,a					; hl = screen address
			ret

;****************************************
; SIMPLE TEXT STRING PRINTER...
;****************************************

; bc = string pointer
; de = hi-res x,y

; codes 0x00 = end

PrintString:
			call	Pixad				; hl = screen address
			ex		de,hl				; de = screen address

CharLoop1:	ld		a,(bc)				; get ascii chr or control code
			or		a
			ret		z

			inc		bc
			cp		0x20				; space chr?
			jr		nz,NotSpace

			inc		e					; skip cell if space
			inc		e
			jr		CharLoop1

NotSpace:	push	bc
			push	de

			sub		48					; number chr?
			cp		10					; 0 to 9?
			jr		c,NotAlpha

			sub		65-48-10			; then must be alpha chr

NotAlpha:	add		a,a					; multiply index by 12
			add		a,a
			ld		l,a
			ld		h,0
			ld		b,h
			add		hl,hl
			ld		c,a
			add		hl,bc

			ld		bc,NUMBERCHARS
			add		hl,bc				; hl = chr data

			ld		b,6					; char is 6 pixels high

ChrLoop2:	ld		a,(hl)
			ld		(de),a
			inc		e
			inc		hl
			ld		a,(hl)
			ld		(de),a
			dec		e
			inc		d
			inc		hl
			djnz	ChrLoop2

			pop		de					; restore screen address
			inc		e					; next cell
			inc		e
			pop		bc					; restore string pointer

			jr		CharLoop1

;****************************************
; NIBBLE PRINT TEXT ROUTINE...
;****************************************

; hl = table
; table = x, y, text, 0

NibblePrinter:

			ld		a,(hl)
			or		a
			ret		z

			ld		e,a
			inc		hl
			ld		d,(hl)
			inc		hl
			ld		c,l
			ld		b,h

			call	NibblePrint

			ld		l,c
			ld		h,b
			inc		hl
			jr		NibblePrinter

; de = coords  bc = string [ = spc

NibblePrint:
			ld		a,0x0f
			bit		2,e
			jr		nz,LowNibble
			cpl
LowNibble:	ld		(NibMask),a

			call	Pixad				; hl = screen address

NibL0:		ld		a,(bc)				; get chr 1
			or		a
			ret		z
			inc		bc

			sub		48					; number chr?
			cp		10					; 0 to 9?
			jr		c,NibAlpha0

			sub		65-48-10			; then must be alpha chr

NibAlpha0:	ld		e,a
			add		a,a
			add		a,a
			add		a,e
			add		a,NibbleChrs & 0xff
			ld		e,a
			ld		d,NibbleChrs / 0x100

			push	bc
			push	hl					; keep screen address

			ld		a,(NibMask)
			ld		c,a
			cpl
			ld		(NibMask),a

			ld		b,5
NibL1:		ld		a,(de)
			and		c	
			or		(hl)
			ld		(hl),a
			inc		e
			call	DownLine
			djnz	NibL1

			pop		hl

			ld		a,c					; need to move a full chr?
			pop		bc
			or		a
			jp		m,NibL0
			inc		l
			jr		NibL0

;****************************************
; END
;****************************************
