;****************************************
;
;	 "SAUCER" - A New Spectrum Game
;
;	Started On The 4th September 2006
;
;
;	(C) 2006 Frobush / Jonathan Smith
;
;****************************************

; Start:
; MainLoop:
; TempSPSetup:
; SetSP:
; GetFX:
; GetMissile:
; GetBaddie:
; FindSP:
; GoSmart:
; WorkSP:
; T_Rover:
; WorkRover:
; Drill:
; T_Saucer:
; T_Wheel:
; T_Wheel_L:
; T_Wheel_R:
; T_BombWait:
; T_BombHeld:
; T_BombFall:
; T_Baddie:
; T_Bottle:
; T_Coin:
; T_CoinOff:
; T_Bullet:
; T_Rocket:
; GoBigBang:
; GoSmallBang:
; T_BigBang:
; T_SmallBang:
; KillSP:
; GoEngineFX:
; T_Engine:	
; T_Arrow:
; CP_16To8:
; CP_8To16:
; CP_16To16:
; OddsEnds:
; BlockVert:
		
;****************************************
; MEMORY MAP...
;****************************************

; 0x4000 - 0x1800 - BITMAP SCREEN
; 0x5800 -  0x300 - ATTRIBUTES
; 0x5B00 - 0x2500 - CODE/GRAPHICS (SLOW)
; 0x8000 - 0xFDFC - CODE/GRAPHICS (FAST)
; 0xFDFD -    0X3 - JP TO INTERRUPTS
; 0xFE00 -  0x101 - INTERRUPT VECTORS
; 0xFF01 -   0xFF - STACK SPACE

; EXECUTION AT 0x8000
; STACK SET TO 0xFFFF

;****************************************
; EQUATES...
;****************************************

STACKADDR:	equ		0xffff				; stack

CODESTART:	equ		0x8000				; execution address

INTTABLE:	equ		0xfe00				; 257 byte vector table
INTVECTOR:	equ		0xfdfd				; jp interrupts 3 bytes at this addrs
SCRNADDR:	equ		0x4000				; screen
ATTRADDR:	equ		0x5800				; attributes
CONTADDR:	equ		0x5e00;0x5b00		; contended RAM (made higher cos of sys vars)

SCOREADDR:	equ		0x51b3				; screen address of score digits
SCANADDR:	equ		0x50c2				; screen address of crystal scanner
COINADDR:	equ		0x5177				; screen address of coin digits

OP_JPHL:	equ		0xe9				; op code for jp (hl)
OP_JP:		equ		0xc3				; op code for jp

BEEPPORT:	equ		0xfe				; beeper output port
BORDERPORT:	equ		0xfe				; border colour output port
KEYPORT:	equ		0xfe				; keyboard input port
JOYPORT:	equ		0x1f				; kempston joystick input port
KEYTOTAL:	equ		0x7					; number of keys to test

SPAREMASK:	equ		0x80				; joypad bits
PAUSEMASK:	equ		0x40
BOMBMASK:	equ		0x20
FIREMASK:	equ		0x10
UPMASK:		equ		0x8
DOWNMASK:	equ		0x4
LEFTMASK:	equ		0x2
RIGHTMASK:	equ		0x1

SPAREBIT:	equ		0x7					; joypad bits
PAUSEBIT:	equ		0x6
BOMBBIT:	equ		0x5					; only these used...
FIREBIT:	equ		0x4
UPBIT:		equ		0x3
DOWNBIT:	equ		0x2
LEFTBIT:	equ		0x1
RIGHTBIT:	equ		0x0

ROWLB:		equ		0xfe				; ROW LEFT BOTTOM
ROWLMB:		equ		0xfd				; ROW LEFT MIDDLE BOTTOM 
ROWLMT:		equ		0xfb				; ROW LEFT MIDDLE TOP
ROWLT:		equ		0xf7				; ROW LEFT TOP
ROWRT:		equ		0xef				; ROW RIGHT	TOP
ROWRMT:		equ		0xdf				; ROW RIGHT	MIDDLE TOP
ROWRMB:		equ		0xbf				; ROW RIGHT	MIDDLE BOTTOM
ROWRB:		equ		0x7f				; ROW RIGHT	BOTTOM

BRIGHT:		equ		0x40				; colour attributes
FLASH:		equ		0x80
PAPER:		equ		0x8					; multply with inks to get paper colour
WHITE:		equ		0x7					; ink colours
YELLOW:		equ		0x6
CYAN:		equ		0x5
GREEN:		equ		0x4
MAGENTA:	equ		0x3
RED:		equ		0x2
BLUE:		equ		0x1
BLACK:		equ		0x0

MENUSELECT:	equ		0x0					; menu options and index
MENUKEYS:	equ		0x1
MENUKEMP:	equ		0x2
MENUSINC:	equ		0x3
MENUCURS:	equ		0x4
MENUREDK:	equ		0x5

RKMENUX:	equ		8*16+4				; centre x for redefinable keys
MENUBOXY:	equ		11
MENUBOXYL:	equ		10	

MAXSCREENX:	equ		0x100				; maximum x screen coord
MAXSCREENY:	equ		0xc0				; maximum y screen coord

WINDOWXL:	equ		8*32				; window width
WINDOWYL:	equ		8*16				; window height
WINDOWATTR:	equ		0 * PAPER + WHITE + BRIGHT		; window attribute
LANDATTR:	equ		0 * PAPER + YELLOW + BRIGHT				; landscape attribute

LOGC:		equ		BLUE

VEC0:		equ		0					; vectors
VECP:		equ		2
VECN:		equ		0xfffe

DEBUGX:		equ		0
DEBUGY:		equ		WINDOWYL + 8

GROUND:		equ		0x1000				; lowest map position
GROUNDHT:	equ		0x10				; amount of ground to show

S_ACCEL:	equ		0x20				; saucer acceleration rate
SB_ACCEL:	equ		0x18				; saucer with bomb acceleration rate
R_ACCEL:	equ		0x8					; rover acceleration rate
SAUCERX:	equ		0x800				; saucer start coords
SAUCERY:	equ		GROUND - 0X20

RELEASEDEL:	equ		0x10				; wheel release delay
WHEELXOFF:	equ		0x0a				; wheels to saucer offsets
WHEELYOFF:	equ		0x0a - 1
BOMBYOFF:	equ		0x10				; bomb y offset
DRILLSIZE:	equ		48					; timers / gauge size

ROVLOCY:	equ		0x11				; rover locator y pos
BOMLOCY:	equ		0x12				; bomb locator y pos
SPRLOCY:	equ		0x13				; spare locator y pos
ROVRIGHT:	equ		8 * 8				; right side of rover (for bomb)

FXOFF:		equ		0					; FX control codes
FXBEEP:		equ		1
FXBOOP:		equ		2
FXHISS:		equ		3
FXGAP:		equ		4

LASTFXCODE:	equ		FXGAP+1

;****************************************

; 1st 16 SPRITES ARE RESERVED...

; 1 X SAUCER
; 8 X MISSILES
; 1 X BOMB
; 2 X WHEEL / OTHER

; 4 X FX (no collisions)

; ALL OTHERS ARE BADDIES

SPTOTAL:	equ		32					; total number of sprites on map
STARTOTAL:	equ		8;16;32				; total number of stars

FXTOTAL:	equ		4
MISSILETOTAL:	equ	6

TYP:		equ		0					; sprite type index (byte) 0 = off
XNO:		equ		TYP + 1				; sprite map x coord (word)
YNO:		equ		XNO + 2				; sprite map y coord (word)
GNO:		equ		YNO + 2				; sprite graphic index (byte)
FLG1:		equ		GNO + 1				; sprite flag1 (word)
FLG2:		equ		FLG1 + 2			; sprite flag2 (word)
NEXTSPVAR:	equ		FLG2 + 2
SPVARLEN:	equ		NEXTSPVAR			; length of sprite data

T_OFF:		equ		0x00				; sprite type indexes - TYP
T_SAUCER:	equ		0x01
T_ROVER:	equ		0x02
T_WHEEL:	equ		0x03
T_WHEELL:	equ		0x04
T_WHEELR:	equ		0x05
T_BOMBWAIT:	equ		0x06
T_BOMBHELD:	equ		0x07
T_BOMBFALL:	equ		0x08
T_FREE:		equ		0x09				; unused
T_BADDIE:	equ		0x0a
T_BOTTLE:	equ		0x0b

T_COIN:		equ		0x0c
T_COINOFF:	equ		0x0d
T_BIGBNG:	equ		0x0e

T_NOCP:		equ		T_COIN				; everything above this has no missile collision

T_BULLET:	equ		0x0f
T_SMALLBNG:	equ		0x10
T_ENGINE:	equ		0x11
T_ARROW:	equ		0x12
T_ROCKET:	equ		0x13

MISSILE:	equ		T_BULLET			; everything below this is a normal sized sprite

G_SAUCER:	equ		0x00				; sprite graphic indexes - GNO
G_WHEEL:	equ		0x01
G_BUBBLE:	equ		0x02
G_BOMB:		equ		0x03
G_BOTTLE:	equ		0x04
G_COIN:		equ		0x05
G_EXPL_R:	equ		0x06
G_EXPL_L:	equ		0x07
G_WHEEL_1:	equ		0x08
G_WHEEL_2:	equ		0x09

M_BUBBLE:	equ		0x00				; 'missile' graphic indexes - GNO
M_BUBWOB:	equ		0x01
M_EXPL_R:	equ		0x02
M_EXPL_L:	equ		0x03
M_MISS_R:	equ		0x04
M_MISS_L:	equ		0x05
M_MISS_U:	equ		0x06
M_ARROW:	equ		0x07
M_DIAMOND:	equ		0x08
M_ORB:		equ		0x09

;****************************************
; CONTENDED MEMORY STUFF...
;****************************************

			org		CONTADDR			; slow ram start

			jp		PAGE0				; for .SNA to work!

			include "source/contended.asm"

CONTENDEND:

CONTENDE:	equ		CONTENDEND - CONTADDR
			defm	CODESTART-CONTADDR-CONTENDE

;****************************************
; PAGE BOUNDRY VARIABLES / TABLES
;****************************************

;****************************************
PAGE0:		jp		Start				; fast ram start
;****************************************

StarVars:	defm	STARTOTAL*2			; star screen x,y

T_SPData:
T_TYP:		defb	0					; temp storage space for sprite vars
T_XNO:		defw	0
T_YNO:		defw	0
T_GNO:		defb	0
T_FLG1:		defw	0
T_FLG2:		defw	0
; more...

Vars:

Temp:		defb	0
TempX:		defw	0
TempY:		defw	0
TempHL:		defw	0
TempHL2:	defw	0
Debug16:	defw	0

Loops:		defw	0					; increased every mainloop
SpeedFlag:	defb	0					; timer flag every interrupt
FUDLR:		defb	0					; bomb,fire,up,down,left,right bits
OldFUDLR:	defb	0					; used for debounce

Carrying:	defb	0					; ID of carried object
Accel:		defb	0					; saucer acceleration
XSpeed:		defb	0
XDir:		defb	0
YSpeed:		defb	0
YDir:		defb	0
ReleaseDel:	defb	0
DrillDel:	defb	0
DrillGag:	defb	0
Crystals:	defb	0					; total number of crystals
Coins:		defb	0					; total number of coins (0 to 99)
CoinDel:	defb	0					; flash coin timer

WindowX:	defw	0					; top left of screen on the map
WindowY:	defw	0
LandHL:		defw	0					; used for landscape screen address
SmartFX:	defw	0					; screen fill word
SmartDel:	defb	0					; smart bomb timer
FreezeLR:	defb	0					; freeze l/r keys

Duf:		defb	0					; general store
NibMask:	defb	0					; nibble mask for small print
AttrFlag:	defb	0					; attribute timer / flag
AttrFlash:	defw	0					; attribute flash table pointer
AttrHL:		defw	0					; attribute screen address store
AttrSTR:	defw	0					; attribute string pointer

FXNumber:	defb	0					; current sound fx number
FXPointer:	defw	0					; pointer to sound data
FXCode:		defw	0					; current beeper routine


VarsEnd:

; these should not be cleared...

StackStore:	defw	0					; temp store for stack

AKBuffer:	defs	"FROBUSH"			; 7 redefine ascii key presses

Rand1:		defb	0xaa				; random numbers
Rand2:		defb	0x55
Rand3:		defb	0xf0

MenuSelect:	defb	0					; menu options (0 is for menu screen)


SPTypes:	defw	T_Off				; sprite type jump pointers
			defw	T_Saucer
			defw	T_Rover
			defw	T_Wheel
			defw	T_Wheel_L
			defw	T_Wheel_R
			defw	T_BombWait
			defw	T_BombHeld
			defw	T_BombFall
			defw	0					; free
			defw	T_Baddie
			defw	T_Bottle
;			- no missile collisions -
			defw	T_Coin
			defw	T_CoinOff
			defw	T_BigBang
;			- missiles -
			defw	T_Bullet
			defw	T_SmallBang
			defw	T_Engine
			defw	T_Arrow
			defw	T_Rocket

SPGnos:		defw	SP_Saucer			; 0x00
			defw	SP_Wheel_1			; 0x01	UNUSED!
			defw	SP_Bubble			; 0x02
			defw	SP_Bomb1			; 0x03
			defw	SP_Bottle			; 0x04
			defw	SP_Coin				; 0x05
			defw	SP_Expl_R			; 0x06
			defw	SP_Expl_L			; 0x07
			defw	SP_Wheel_1			; 0x08
			defw	SP_Wheel_2			; 0x09

SPSGnos:	defw	SPS_Bubble			; 0x00
			defw	SPS_BubbleWobble	; 0x01
			defw	SPS_Expl_R			; 0x02
			defw	SPS_Expl_L			; 0x03
			defw	SPS_Missile_R		; 0x04
			defw	SPS_Missile_L		; 0x05
			defw	SPS_Missile_U		; 0x06
			defw	SPS_Arrow			; 0x07
			defw	SPS_Diamond			; 0x08
			defw	SPS_Orb				; 0x09

SP3Offs:	defw	8*2*3*0,8*2*3*1,8*2*3*2,8*2*3*3		; 3 * (2 * 8)

SP2Offs:	defw	8*1*2*0,8*1*2*1,8*1*2*2,8*1*2*3		; 2 * (1 * 8)

;****************************************

PAGE0END:
PAGE0E:		equ		PAGE0END - PAGE0
			defm	256 - PAGE0E

;****************************************
PAGE1:
;****************************************

BitTable:	defb	0x80,0x40,0x20,0x10,0x8,0x4,0x2,0x1

KeyJoy2Vec:	defw	VEC0,VECP,VECN,VEC0

BladeTab:	defw	RotorShape1			; bomb blade image pointers
			defw	RotorShape2
			defw	RotorShape3
			defw	RotorShape4

Num2Vec16:	defw	VEC0,VECN,VECP,VECN
			defw	VECP,VEC0,VECP,VECP
			defw	VEC0,VECP,VECN,VECP
			defw	VECN,VEC0,VECN,VECN

Num2Vec8:	defb	0x00,0xfe,0x02,0xfe
			defb	0x02,0x00,0x02,0x02
			defb	0x00,0x02,0xfe,0x02
			defb	0xfe,0x00,0xfe,0xfe

KeySwap:	defb	6,4,2,0,8,10,12,14	; bodge index 'cos of UDLRF / LRUBF

KeyTable:
			defb	ROWRMT,0x1			; P	redefineable - these are defaults
			defb	ROWRMT,0x2			; O
			defb	ROWLMB,0x1			; A
	        defb	ROWLMT,0x1			; Q
			defb	ROWRB,0x4			; M
			defb	ROWRB,0x8			; N
			defb	ROWRMB,0x1			; ENTER	for pause

SinclairTable:
			defb	ROWRT,0x8			; 7
			defb	ROWRT,0x10			; 6
			defb	ROWRT,0x4			; 8
	        defb	ROWRT,0x2			; 9
			defb	ROWRT,0x1			; 0
			defb	ROWRB,0x1			; SPACE
			defb	ROWRMB,0x1			; ENTER	for pause

CursorTable:
			defb	ROWRT,0x4 			; 8
			defb	ROWLT,0x10			; 5
			defb	ROWRT,0x10			; 6
	        defb	ROWRT,0x8			; 7
			defb	ROWRT,0x1			; 0
			defb	ROWRB,0x1			; SPACE
			defb	ROWRMB,0x1			; ENTER	for pause

MenuTable:
			defb	ROWLT,0x10 			; 5
			defb	ROWLT,0x8			; 4
			defb	ROWLT,0x4			; 3
	        defb	ROWLT,0x2			; 2
			defb	ROWLT,0x1			; 1
			defb	ROWRB,0x1			; SPACE	(stop music?)
			defb	ROWRMB,0x1			; ENTER	for pause

Menu2KeyTable:
			defb	MenuTable & 0xff
			defb	KeyTable & 0xff
			defb	0					; kempston
			defb	SinclairTable & 0xff
			defb	CursorTable & 0xff

SauXMove:	defw	SauSlowX			; saucer move pointers
			defw	SauRight
			defw	SauLeft
			defw	SauSlowX

SauYMove:	defw	SauSlowY
			defw	SauRight
			defw	SauLeft
			defw	SauSlowY
					 
CTab:		defb    0x00,0x00,0x01,0x01,0x01,0x02,0x02,0x03
			defb    0x03,0x03,0x04,0x04,0x04,0x05,0x05,0x05
			defb    0x06,0x06,0x06,0x07,0x07,0x07,0x08,0x08
			defb    0x08,0x09,0x09,0x09,0x09,0x0A,0x0A,0x0A
			defb    0x0B,0x0B,0x0B,0x0B,0x0C,0x0C,0x0C,0x0C
			defb    0x0C,0x0D,0x0D,0x0D,0x0D,0x0D,0x0E,0x0E
			defb    0x0E,0x0E,0x0E,0x0E,0x0E,0x0E,0x0F,0x0F
			defb    0x0F,0x0F,0x0F,0x0F,0x0F,0x0F,0x0F,0x0F

QuadX:		defb	0x00
QuadY:		defb	0x00,0xFF,0xFF
QuadTab:	defb	0x00,0x3F,0x00,0x3F

			defb	0					; score overflow - reset every now and again!
Score:		defs	"00000000"			; the score ascii string

;****************************************

PAGE1END:
PAGE1E:		equ		PAGE1END - PAGE1
			defm	256 - PAGE1E

;****************************************
PAGE2:
;****************************************

NibbleFXTab:
			defw	AttrStrBonus
			defw	AttrStrHurry
			defw	AttrStrBang
			defw	AttrStrDropIt

NibbleChrs:
			defb	 68,170,170,170, 68,  136, 68	; 0 to 9 - 1 * 5 * 36
			defb	 68, 68,238,  204, 34, 68,136
			defb	238,  238, 34, 68, 34,204 
			defb	170,170,102, 34, 34,  238,136
			defb	 68, 34,204,  102,136,204,170
			defb	204,  238, 34, 34, 34, 34
			defb	 68,170, 68,170, 68,  102,170
			defb	102, 34, 34
			defb	 68,170,238,170,170,204,170		; a to z
			defb	204,170,238,102,136,136,136
			defb	238,204,170,170,170,238
			defb	238,136,204,136,238,238,136
			defb	204,136,136,102,136,170,170
			defb	238,170,170,238,170,170
			defb	238, 68, 68, 68,238, 34, 34
			defb	 34,170,102,136,170,204,170
			defb	170,136,136,136,136,238
			defb	170,238,170,170,170
			defb	204,170,170,170,170, 68,170
			defb	170,170, 68,204,170,204,136
			defb	136,102,170,102, 34, 34
			defb	204,170,204,170,170,102,136
			defb	 68, 34,204,238, 68, 68, 68
			defb	 68,170,170,170,170,102
			defb	170,170,170,170, 68,170,170
			defb	170,238,170,170,170, 68,170
			defb	170
			defb	170,170, 68, 68, 68				; "y"
			defb	238, 34, 68,136,238
			defb	0,0,0,0,0						; [ = space
			defb	0,0,0,0,68						; \	= "."
			defb	204,34,68,0,68					; ] = "?"
			defb	34,68,68,68,34					; ^ = "("
			defb	136,68,68,68,136				; _ = ")"
	
;****************************************

PAGE2END:
PAGE2E:		equ		PAGE2END - PAGE2
			defm	256 - PAGE2E

;****************************************
PAGE3:
;****************************************

WindowDl:	defm	WINDOWYL			; for screen print

FXTEST:		equ		0x01
FXRKON:		equ		0x02				; rkeys
FXRKCLICK:	equ		0x03				; rkeys
FXBANG1:	equ		0x04
FXBANG2:	equ		0x05
FXBANG3:	equ		0x06
FXBANG4:	equ		0x07
FXFIRE1:	equ		0x08
FXFIRE2:	equ		0x09
FXFIRE3:	equ		0x0a
FXFIRE4:	equ		0x0b
FXLINK1:	equ		0x0c
FXUNLINK1:	equ		0x0d
FXUNLINK2:	equ		0x0e
FXCOIN:		equ		0x0f

FXRout:		defw	0					; 0
			defw	Beep				; 1 normal beep
			defw	Boop				; 2	phase beep
			defw	Hiss				; 3	white noise
			defw	WaitInt				; 4 no sound

FXTable:	defw	TestFX				; 0x01
			defw	RKOnFX				; 0x02
			defw	RKClickFX			; 0x03
			defw	BangFX1				; 0x04
			defw	BangFX2				; 0x05
			defw	BangFX3				; 0x06
			defw	BangFX4				; 0x07
			defw	FireFX1				; 0x08
			defw	FireFX2				; 0x09
			defw	FireFX3				; 0x0a
			defw	FireFX4				; 0x0b
			defw	LinkFX1				; 0x0c
			defw	UnLinkFX1			; 0x0d
			defw	UnLinkFX2			; 0x0e
			defw	CoinFX				; 0x0f

LandBlocks:	defw	Block08				; 0x00 diamond
			defw	Block09
			defw	Block00
			defw	Block01
			defw	Block02
			defw	Block03
			defw	Block04
			defw	Block05
			defw	Block06
			defw	Block07

LandShift:	defw	3*32,2*32,1*32,0*32

LandAtTab:	defb	WINDOWATTR,WINDOWATTR	; landscape attrs
			defb	WINDOWATTR,WINDOWATTR
			defb	WINDOWATTR,LANDATTR
			defb	LANDATTR,LANDATTR

DiamATTR:	defb	6 * PAPER + LANDATTR	; diamond flash table
			defb	0 * PAPER + LANDATTR
			defb	6 * PAPER + LANDATTR
			defb	0 * PAPER + LANDATTR
			defb	4 * PAPER + LANDATTR
			defb	0 * PAPER + LANDATTR
			defb	6 * PAPER + LANDATTR
			defb	0 * PAPER + LANDATTR
/*
DiamATTR:	defb	7 * PAPER + LANDATTR	; diamond flash table
			defb	0 * PAPER + LANDATTR
			defb	7 * PAPER + LANDATTR
			defb	0 * PAPER + LANDATTR
			defb	5 * PAPER + LANDATTR
			defb	0 * PAPER + LANDATTR
			defb	7 * PAPER + LANDATTR
			defb	0 * PAPER + LANDATTR
*/
RovAAts:	defw	AttrARovB			; rover locater attr pointers
			defw	AttrARovL
			defw	AttrARovR

BomAAts:	defw	AttrARovB			; bomb locater attr pointers
			defw	AttrABomL
			defw	AttrABomR

SprAAts:	defw	AttrARovB			; spare locater attr pointers
			defw	AttrASprL
			defw	AttrASprR

WeapAttrAd:	defw	17 * 0x20 + ATTRADDR + 25	; 4 weapon monitors?
			defw	18 * 0x20 + ATTRADDR + 28
			defw	21 * 0x20 + ATTRADDR + 28
			defw	21 * 0x20 + ATTRADDR + 25

			defw	0x5039				; and their screen addresses
			defw	0x505c
			defw	0x50bc
			defw	0x50b9

WeapAttr:	defw	0 * PAPER + WHITE,0 * PAPER + WHITE + BRIGHT

;****************************************

PAGE3END:
PAGE3E:		equ		PAGE3END - PAGE3
			defm	256 - PAGE3E

;****************************************
PAGE4:
;****************************************

Land:		nop							; full page!

;****************************************

PAGE4END:
PAGE4E:		equ		PAGE4END - PAGE4
			defm	256 - PAGE4E

;****************************************

SPData:		defm	SPTOTAL * SPVARLEN	; sprite data
SPDataEnd:	defb	0xff				; THIS MUST BE 0xff!
/*
SAUCERVARS:		equ		SPVARLEN * 0x00 + SPData		; 1
MISSILEVARS:	equ		SPVARLEN * 0x01 + SPData		; 8
BOMBVARS:		equ		SPVARLEN * 0x09 + SPData		; 1
WHEELVARS:		equ		SPVARLEN * 0x0a + SPData		; 2
FXVARS:			equ		SPVARLEN * 0x0c + SPData		; 4
BADDIEVARS:		equ		SPVARLEN * 0x10 + SPData		; rest
*/
SAUCERVARS:		equ		SPVARLEN * 0x00 + SPData		; 1
MISSILEVARS:	equ		SPVARLEN * 0x01 + SPData		; 6
BOMBVARS:		equ		SPVARLEN * 0x07 + SPData		; 1
WHEELVARS:		equ		SPVARLEN * 0x08 + SPData		; 2
FXVARS:			equ		SPVARLEN * 0x0a + SPData		; 4
BADDIEVARS:		equ		SPVARLEN * 0x0e + SPData		; rest

;****************************************
;****************************************
;****************************************
; GAME START AND SETUP...
;****************************************
;****************************************
;****************************************

Start:		halt						; make it clean
			xor		a
			out		(BORDERPORT),a
			di

			ld		sp,STACKADDR		; set the stack pointer

			call	ScreenOff

			xor		a
			ld		hl,SPData			; clear the sprite data
			ld		de,SPData+1
			ld		bc,SPTOTAL*SPVARLEN-1
			ld		(hl),a
			ldir
			ld		(hl),0xff			; end of vars code!

			ld		hl,Vars				; clear the main variable area
			ld		de,Vars+1
			ld		bc,VarsEnd-Vars-1
			ld		(hl),a
			ldir

			ld		(MenuSelect),a		; set to default MENUSELECT

			ld		hl,SCRNADDR+30		; generate screen y table for window
			ld		de,WindowDl
			ld		b,WINDOWYL/2		; 0x80 bytes (2 pixel positions)
WindowD:	ex		de,hl
			ld		(hl),e
			inc		l
			ld		(hl),d
			inc		l
			ex		de,hl
			inc		h
			call	DownLine
			djnz	WindowD

			ld		hl,Land				; create a random landscape
			ld		b,0
CreateMap:	push	hl
			call	Random
			pop		hl
			ld		a,(Rand2)
			and		7
			jr		z,PlopHole			; diamond hole is blocks 00/01
			ld		a,(Rand1)
			and		0xc					; blocks are in groups of 2s and indexes are * 2
			add		a,2 * 2				; skip diamond blocks
PlopHole:	ld		(hl),a
			inc		l
			inc		a
			inc		a
			ld		(hl),a
			inc		l
			djnz	CreateMap

			call	BlockVert			; create blocks

;****************************************

			ld      hl,Interrupt		; setup interrupt routine
			call	SetupInts
			call	WorkMenu			; do the menu / intro
			call	TempSPSetup
			call	SetStars
			xor		a					; silly me!
			ld		(FUDLR),a
			call	WorkSP				; work all the sprites

			call	WaitVBlank			; wait for flyback
			call	ScreenOff
			call	DrawStatus			; draw the status line

			;*******************
;			call	DebugDisplayOnce
			;*******************

			call	WaitVBlank			; wait for flyback
			ld		hl,AttrBoxesMain
			call	FillAttrBoxes		; screen on!
			call	TeleColour			; blimey!

;****************************************

MainLoop:
			;*******************
;			ld		hl,12345
;			call	Sqr16
;			ld		(Temp),a
;			call	DebugDisplay 		; debug display
			;*******************

			call	WaitVBlank			; wait for flyback
			call	ReadKeyStick		; read keyboard / joysticks
			call	OddsEnds			; general stuff (like constant animation)
			call	Random				; keep generating random numbers
			call	PauseGame
			call	WorkAttrFX			; attribute screen fx
			call	Gauges				; work gauges

			ld		bc,150				; this should be used for other things!
Wait:		dec		bc
			ld		a,b
			or		c
			jr		nz,Wait

			call	DumpScreen			; draw the play area
			call	Stars				; draw stars
			call	DumpSP				; draw the sprites
			call	Landscape			; draw landscape
			call	LandAttr			; colour landscape
			call	WorkSP				; work the sprites

			ld		a,(FUDLR)			; abort?
			and		PAUSEMASK+BOMBMASK+FIREMASK
			cp		PAUSEMASK+BOMBMASK+FIREMASK
			jr		nz,MainLoop			; no abort pressed

			ei
			jp		Start				; start again

;****************************************
; TEMP SPRITE SETUP FOR TESTING...
;****************************************

TempSPSetup:

; saucer...

			ld		hl,SAUCERVARS		; setup the saucer

			ld		de,SAUCERX
			ld		bc,SAUCERY
			ld		a,G_SAUCER			; graphics index
			ex		af,af
			ld		a,T_SAUCER
			call	SetSP

			ld		hl,SAUCERX-0x80
			ld		(WindowX),hl

			ld		a,S_ACCEL			; saucer acceleration
			ld		(Accel),a

;****************************************

; wheels...

			ld		hl,WHEELVARS
			ld		de,SAUCERX - 0x100 - 10
			ld		bc,GROUND
			ld		a,G_WHEEL_1			; graphic index
			ex		af,af
			ld		a,T_WHEEL
			call	SetSP

			ld		hl,FXVARS
			ld		de,SAUCERX - 0x100
			ld		bc,GROUND + 0x10
			ld		a,M_ARROW			; graphic index
			ex		af,af
			ld		a,T_ARROW			; type
			call	SetSP
			ld		(hl),0				; flg1 = delay to stop saucer collision

			ld		hl,WHEELVARS + SPVARLEN
			ld		de,SAUCERX - 0x100 + 10
			ld		bc,GROUND
			ld		a,G_WHEEL_2			; graphic index
			ex		af,af
			ld		a,T_WHEEL
			call	SetSP

;****************************************

			ld		de,SAUCERX - 0x58
			ld		bc,SAUCERY - 0x20
			xor		a
			ld		(Duf),a

SetSPL:		push	de
			call	GetBaddie
			pop		de
			ret		m					; end of sprite vars?

			push	hl

			ld		a,(Duf)
			push	af
;			and		3
;			add		a,G_BUBBLE			; graphics index
			ld		a,G_BUBBLE
			ex		af,af
			ld		a,T_BADDIE
			call	SetSP

			pop		af
			inc		a
			ld		(Duf),a
			ld		(hl),a				; flg1 (vector)

			ld		hl,8*10				; spread them out
			add		hl,de
			ex		de,hl
			ld		a,c
			add		a,8
			ld		c,a

			pop		hl
			push	de
			ld		de,SPVARLEN
			add		hl,de
			pop		de

			jr		SetSPL

;****************************************
; SETUP A SPRITE...
;****************************************

;  a = type
; 'a = gno
; de = x
; bc = y
; hl = SPData

; returns with hl as address of next sprite	var after gno
;								-------------------------

SetSP:		ld		(hl),a				; type
			inc		hl
			ld		(hl),e				; 16 bit x coord
			inc		hl
			ld		(hl),d
			inc		hl
			ld		(hl),c				; 16 bit y coord
			inc		hl
			ld		(hl),b
			inc		hl
			ex		af,af
			ld		(hl),a				; graphic index
			inc		hl
			ret

;****************************************
; FIND SPRITES...
;****************************************

GetFX:		ld		b,FXTOTAL
			ld		hl,FXVARS
			jr		SearchSP

;****************************************

GetMissile:	ld		b,MISSILETOTAL
			ld		hl,MISSILEVARS

SearchSP:	ld		de,SPVARLEN

SearchMis:	ld		a,(hl)
			or		a
			ret		z					; 0 = free sprite
			add		hl,de
			djnz	SearchMis
			or		a
			ret

;****************************************

GetBaddie:	ld		hl,BADDIEVARS

FindSP:		ld		de,SPVARLEN

SearchBad:	ld		a,(hl)
			or		a
			ret		z					; 0 = free sprite
			ret		m					; minus is end - careful here!
			add		hl,de
			jr		SearchBad

;****************************************
; Smart Bomb - Kill All On Screen...

GoSmart:	ld		hl,BADDIEVARS
			ld		de,SPVARLEN
			ld		bc,GNO

SearchOn:	ld		a,(hl)				; typ
 			or		a
  			ret		m					; no more

   			jr		z,NextOn			; is dead

			cp		T_NOCP				; has no collision
			jr		nc,NextOn

			add		hl,bc
 			ld		a,(hl)				; gno (bit 7 set if on screen)
  			sbc		hl,bc
   			or		a
			jp		p,NextOn			; ignore if not on screen

			push	hl
 			ld		(hl),T_BIGBNG		; big explosion
  			add		hl,bc
   			ld		(hl),G_EXPL_R		; explosion graphic
			inc		hl
			ld		(hl),8				; timer
			push	de
			ld		de,0x0105			; 100 pts - 1st is num 2nd is index
			call	ScoreAdd			; upscore
			pop		de
			pop		hl

NextOn:		add		hl,de
			jr		SearchOn

;****************************************
; WORK SPRITES...
;****************************************

WorkSP:		ld		hl,SPData			; sprite data pointer

			ld		a,(FreezeLR)		; freeze l/r controls?
			or		a
			jr		z,WorkSPLoop

			dec		a
			ld		(FreezeLR),a

			ld		a,(FUDLR)
			and		0xfc
			ld		(FUDLR),a

WorkSPLoop:	ld		a,(hl)				; type index (or 0xff = end of vars)
			add		a,a
			ret		c					; can only have 0x7f types!

			push	hl

			ld		de,T_SPData			; temp sprite data store
			ldi		; TYP
			ldi		; XNO LO
			ldi		; XNO HL
			ldi		; YNO LO
			ldi		; YNO HI
			ldi		; GNO
			ldi		; FLG1 LO
			ldi		; FLG1 HO
			ldi		; FLG2 LO
			ldi		; FLG2 HO

			add		a,SPTypes &	0xff
			ld		l,a
			ld		h,SPTypes / 0x100
			call	JumpHL

			pop		de
			ld		hl,T_SPData			; temp sprite data store

			ldi		; TYP
			ldi		; XNO LO
			ldi		; XNO HL
			ldi		; YNO LO
			ldi		; YNO HI
			ldi		; GNO
			ldi		; FLG1 LO
			ldi		; FLG1 HO
			ldi		; FLG2 LO
			ldi		; FLG2 HO

			ex		de,hl
			jr		WorkSPLoop

JumpHL:		ld		a,(hl)
			inc		l
			ld		h,(hl)
			ld		l,a
			defb	OP_JPHL				; jp (hl)

;****************************************

T_Off:		ret

;****************************************
; WORK THE ROVER...
;****************************************

T_Rover:	ld		a,(XSpeed)			; bubbles out of engines?
			or		a
			jr		z,NoBubs

			srl		a
			srl		a
			srl		a
			srl		a
			add		a,-16
			ld		c,a
			ld		b,0xff				; bc = y offset
			ld		a,M_BUBBLE			; graphic index
			call	GoEngineFX

			xor		a					; kill the drill
			ld		(DrillDel),a

NoBubs:		ld		a,(FUDLR)

			;****************

			; Fire Weapons?

			bit		FIREBIT,a			; fire a missile?
			jr		z,NoFire0
			push	af
			ld		a,(OldFUDLR)
			bit		FIREBIT,a
			jr		nz,NoFir0

			call	GetMissile			; free missile sprite?
			jr		nz,NoFir0

			;****************

			; Setup Rocket...

			push	hl
			ld		de,(T_XNO)			; above rover
			ld		hl,(T_YNO)
			ld		bc,-16
			add		hl,bc
			ld		c,l
			ld		b,h
			ld		a,M_MISS_U			; graphic index
			ex		af,af
			ld		a,T_ROCKET			; type
			pop		hl
			call	SetSP
			inc		l
			ld		(hl),18;16			; flg1 hi = timer
			ld		a,(Rand1)
			and		3
			add		a,FXFIRE1			; go sound fx
			call	GoFX

NoFir0:		pop		af

NoFire0:	ld		c,a
			and		UPMASK				; pushing up?
			jr		nz,Back2Sau

			ld		a,RELEASEDEL		; keep setting wheel release delay
			ld		(ReleaseDel),a
			jp		WorkRover

			;****************

			; Lift Off?

Back2Sau:	ld		a,c					; slow down
			and		0xfc				; by removing left and right keypresses
			ld		c,a
			ld		a,(XSpeed)			; have we stopped?
			or		a
			jp		nz,WorkRover

			ld		a,(ReleaseDel)		; ready to lift off?
 			dec		a
   			jr		z,LiftOff

  			ld		(ReleaseDel),a		; do a sound FX?
			ld		b,a
			dec		a
			dec		a
			and		2
			jr		nz,NoLiftFX

			ld		a,b
			add		a,LASTFXCODE + 1	; beware codes!
			add		a,a
			add		a,a
			ld		(UnLinkFX1 + 1),a
			ld		a,FXUNLINK1			; go sound fx
			call	GoFX

NoLiftFX:	call	GetFX				; free FX sprite?
			jp		nz,WorkRover

			;****************

			; Countdown...

			push	bc					; setup booster smoke...
			push	hl

			ld		a,(Rand1)			; random coords around saucer...
			and		0xf
			ld		e,a
			ld		hl,0x6
			ld		d,h
			sbc		hl,de
			ld		de,(T_XNO)
			add		hl,de
			ex		de,hl

			ld		bc,GROUND - 0x8
			ld		a,(Rand2)
			and		0x7
			add		a,c
			ld		c,a

			ld		a,G_EXPL_R			; graphic index
			ex		af,af
			ld		a,T_BIGBNG			; type
			pop		hl
			call	SetSP
			ld		(hl),0x6			; timer
			pop		bc
   			jr		WorkRover

			;****************

			; Lift Off...

LiftOff:	ld		a,T_WHEEL			; change to normal wheels
			ld		(WHEELVARS),a
			ld		(WHEELVARS + SPVARLEN),a

			xor		a
			ld		(DrillDel),a		; reset drill delay

			ld		a,S_ACCEL			; saucer acceleration
			ld		(Accel),a

			ld		a,T_SAUCER			; back to saucer
			ld		(T_TYP),a
			ld		hl,0xffff			; max y speed
			ld		(YSpeed),hl
			ld		a,l					; upwards
			ld		(YDir),a

			ld		hl,FXVARS			; setup arrow pointer (steal FX sprite)
			ld		de,(T_XNO)
			ld		bc,GROUND + 0x10
			ld		a,M_ARROW			; graphic index
			ex		af,af
			ld		a,T_ARROW			; type
			call	SetSP
			ld		(hl),10				; flg1 = collision delay

			ld		a,FXBANG1			; go sound fx
			jp		GoFX

			;****************

			; Drive The Rover...

WorkRover:	ld		a,c
			add		a,a
			and		6
			add		a,SauXMove & 0xff
			ld		l,a
			ld		h,SauXMove / 0x100

			ld		bc,(XSpeed)			; c = xspeed b = xdir
;			ld		e,S_ACCEL / 4
		ld		a,(Accel)
		ld		e,a
			call	JumpHL
			ld		c,a
			ld		(XSpeed),bc			; keep b for later

			srl		a
			srl		a
			srl		a
			ld		c,a					; for window positioning
			srl		a
			ld		e,a
			ld		d,0
			ld		hl,(T_XNO)			; add x vector to x
			jr		z,NoXSpdR

			inc		e
			inc		b
			jr		nz,SaLR
			ld		a,e
			neg
			ld		e,a
			jp		p,SaLR
			dec		d
SaLR:		add		hl,de
			res		0,l
			ld		(T_XNO),hl			; saucer map x coord

			;****************

			; Rover Window Movement...

NoXSpdR:	ld		de,WINDOWXL / 2
			or		a
			sbc		hl,de				; place saucer in centre
			or		a
		 	jr		z,NoWMov
			dec		b
			jr		z,WinMovX

			ld		a,c
 			neg
  			ld		c,a
   			jp		m,WinMovX
			inc		b
WinMovX:	add		hl,bc				; move window to show area infront of saucer

NoWMov:		ld		(WindowX),hl		; new window x coord

			ld		hl,GROUND - WINDOWYL + GROUNDHT
			ld		(WindowY),hl		; window at bottom of map

			ld		hl,GROUND - WHEELYOFF	; height mapped?
			ld		(T_YNO),hl

			;****************

			; Over A Diamond...

			ld		hl,(T_XNO)			; look under the rover
			call	GroundAddr
			or		a
			ret		nz					; ingore if it's not a diamond

			push	hl
			ld		a,(Loops)			; flash the diamond...
			and		7
			add		a,DiamATTR & 0xff
			ld		l,a
			ld		h,DiamATTR / 0x100
			ld		a,(hl)
			ld		hl,15 * 0x20 + ATTRADDR + 14
			ld		e,l
			ld		d,h
			inc		e
			ld		(hl),a
			ldi
			ldi
			ldi
			pop		hl

			ld		a,(XSpeed)			; we must be still to work the drill
			or		a
			ret		nz
			ld		a,(FUDLR)			; and pressing down
			and		DOWNMASK
			jr		nz,Drill

			ld		(DrillDel),a		; reset drill delay
			ret

			;****************

Drill:		ld		a,(DrillGag)
			ld		c,a
			ld		a,(DrillDel)		; drill it up yet?
			cp		c
			ret		c					; must wait for gauge to fall

			inc		a
			cp		DRILLSIZE
			jr		nz,DrillAni

			xor		a
			ld		(DrillDel),a		; reset drill delay

			ld		(hl),2*2			; eat diamond!
			inc		l
			ld		(hl),3*2
			ld		a,l					; put back elsewhere
			add		a,0x80
			ld		l,a
			ld		(hl),1*2
			dec		l
			ld		(hl),0*2

			ld		a,(Crystals)		; another crystal
			inc		a
			ld		(Crystals),a

			ld		de,0x0205			; 200 pts - 1st is num 2nd is index
			call	ScoreAdd			; upscore

			ld		a,FXLINK1			; go sound fx
			jp		GoFX

			;****************

DrillAni:	ld		(DrillDel),a		; animate the dig
			ld		c,a
			ld		a,0xfe				; suck up the crystal
			sub		c
			sub		c
			sub		c
			ld		(UnLinkFX1 + 1),a

			ld		a,(Loops)
			and		1
			ret		nz
			ld		hl,(T_YNO)
			dec		hl
			dec		hl
			ld		(T_YNO),hl

			ld		bc,16				; bc = y offset
			ld		a,M_DIAMOND			; graphic index
			call	GoEngineFX

			ld		a,FXUNLINK1			; go sound fx
			jp		GoFX

;****************************************
; WORK THE SAUCER...
;****************************************

;  c = XSpeed
;  b = Xdir 0right or <>0left
;  e = S_ACCEL

SauRight:	ld		a,c
			bit		7,b
			jr		nz,SlowR
			add		a,e
			ret		nc
			ld		a,0xff				; max speed
			ret			
SlowR:		sub		e
			ret		nc
			xor		a					; stop
			ld		b,a					; can now move right
			ret

			;****************

SauLeft:	ld		a,c
			bit		7,b
			jr		z,SlowL
			add		a,e
			ret		nc
			ld		a,0xff				; max speed
			ret			
SlowL:		sub		e
			ret		nc
			xor		a					; stop
			ld		b,0xff				; left
			ret

			;****************

SauSlowX:	ld		a,c
			sub		e
			ret		nc
			xor		a					; stop
			ret

			;****************

SauSlowY:	ld		a,(Carrying)		; heavy if carrying something
			or		a
			ld		a,c
  			jr		z,NotHeavy

			bit		7,b
			jr		nz,Heavy
			add		a,e
			ret		nc
			ld		a,0xff				; max speed
			ret			

Heavy:		sub		e
			ret		nc
			xor		a					; stop
			ld		b,a					; can now move down
			ret

NotHeavy:	sub		e
			ret		nc
			xor		a					; stop
			ret

;****************************************

T_Saucer:	ld		a,(XSpeed)
			ld		c,a
			ld		a,(YSpeed)
			or		c
			ld		a,M_EXPL_R			; graphic index
			ld		bc,4				; y offset
			call	nz,GoEngineFX

			ld		a,(FUDLR)

			;****************

			; Fire Weapons?

			bit		FIREBIT,a			; fire a missile?
			jr		z,NoFire
			push	af
			ld		a,(OldFUDLR)
			bit		FIREBIT,a
			jr		nz,NoFir

			call	GetMissile			; free missile sprite?
			jr		nz,NoFir

			;****************

			; setup bullet

			push	hl
			ld		hl,(T_XNO)			; from saucer - which side?
			ld		de,6
			ld		a,(XDir)
			or		a
			jr		nz,LeftSide
			add		hl,de
			jr		OffSide
LeftSide:	sbc		hl,de
OffSide:	ex		de,hl

			pop		hl
			ld		bc,(T_YNO)
			ld		a,T_BULLET			; ignore graphic - setup later
			call	SetSP
			ld		a,(XDir)
			or		a
			ld		a,0xa
			ld		c,M_MISS_R			; graphics index
			jr		z,NFir
			neg
			inc		c
NFir:		dec		hl
			ld		(hl),c				; graphic
			inc		hl
			ld		(hl),a				; vector
			inc		hl
			ld		(hl),25				; timer
			ld		a,(Rand1)
			and		3
			add		a,FXFIRE1			; go sound fx
			call	GoFX

NoFir:		pop		af

			;****************

			; Fly The Saucer...

NoFire:		push	af
			add		a,a
			and		6
			add		a,SauXMove & 0xff
			ld		l,a
			ld		h,SauXMove / 0x100

			ld		bc,(XSpeed)			; c = xspeed b = xdir
;			ld		e,S_ACCEL
		ld		a,(Accel)
		ld		e,a
			call	JumpHL
			ld		c,a
			ld		(XSpeed),bc			; keep b for later

			srl		a
			srl		a
			srl		a
			srl		a
			srl		a
			ld		c,a					; for window positioning
			ld		e,a
			ld		d,0
			ld		hl,(T_XNO)			; add x vector to x
			jr		z,NoXSpd

			inc		e
			inc		b
			jr		nz,SaL
			ld		a,e
			neg
			ld		e,a
			jp		p,SaL
			dec		d
SaL:		add		hl,de
			res 	0,l
			ld		(T_XNO),hl		; saucer map x coord

NoXSpd:		ld		de,WINDOWXL / 2
			or		a
			sbc		hl,de
			ld		(WindowX),hl		; window x coord

			;*******************

			pop		af
			srl		a
			and		6
			add		a,SauYMove & 0xff
			ld		l,a
			ld		h,SauYMove / 0x100

			ld		bc,(YSpeed)			; c = Yspeed b = Ydir
;			ld		e,S_ACCEL
		ld		a,(Accel)
		ld		e,a
			call	JumpHL
			ld		c,a
			ld		(YSpeed),bc

			srl		a
			srl		a
			srl		a
			srl		a
			srl		a
			ld		c,a					; for window positioning
			ld		e,a
			ld		d,0

			ld		hl,(T_YNO)			; add y vector to y
			inc		b
			jr		nz,SaU
			or		a
			jr		z,RoofCP
			cpl
			ld		e,a
			dec		d
SaU:		add		hl,de				; hl = new y pos

			;*******************

			; bounce off top of map?

RoofCP:		ld		a,h					; h is 0 if at top part of map
			or		a
			jr		nz,GroundCP
			ld		a,l
			cp		0x10				; saucer size?
			jr		nc,GroundCP
			ld		hl,0x10				; restore y if at top of map

			xor		a					; bounce down
			jr		YBounce

			;*******************

			; Carrying Object - And Object Hits Ground?

GroundCP:	ld		a,(Carrying)		; ID of object being carried
			or		a
			jr		z,NotCarry

			push	hl
			ld		de,0x10				; could be different size?
			add		hl,de
			ld		a,h
			pop		hl
			cp		GROUND / 0x100		; object hit ground?
			jr		c,NoMoreYCP

			ld		hl,GROUND - 0x10	; place object + saucer on floor
			jr		YBouUp

			;*******************

			; Bounce on Ground?

NotCarry:	ld		a,h
			cp		GROUND / 0x100		; hit ground?
			jr		c,NoMoreYCP
			ld		hl,GROUND			; place on floor

YBouUp:		ld		a,0xff				; bounce up

YBounce:	ld		(YDir),a
			ld		a,(YSpeed)			; slow down
			srl		a
			ld		(YSpeed),a
			ld		c,a					; needed?

			;*******************

			; Place Saucer In Centre Of Screen...

NoMoreYCP:	res		0,l

			ld		(T_YNO),hl			; saucer map y coord

WindYSet:	ld		de,WINDOWYL/2 + 0x10
			or		a
			sbc		hl,de				; place saucer in centre

			dec		b
			jr		z,WinMovY
			ld		a,c
			cpl
			ld		c,a
WinMovY:	ld		a,c
			and		0xfe
			add		hl,bc				; move window to show area infront of saucer

			ld		a,h					; window off top of map?
			or		a
			jp		p,WinNotTop
			ld		hl,0				; place window at top of map

WinNotTop:	ld		(WindowY),hl		; new window x coord

			ld		de,GROUND - WINDOWYL + GROUNDHT
			or		a
			sbc		hl,de
			ret		c

			ld		(WindowY),de		; window at bottom of map
			ret

;****************************************
; WHEELS...
;****************************************

T_Wheel:	
;			ld		hl,12345
;			call	Sqr16
;			ld		(Temp),a


/*
			ld		a,(T_GNO)
			and		127
			cp		G_WHEEL_1			; graphic index
			ret		nz

			ld		a,(Loops)
			add		a,a
			add		a,a
			add		a,a
			call	Circle
			ld		hl,(SAUCERVARS+XNO)
			add		hl,de
;			add		hl,de
;			add		hl,de
			ld		a,l
			ld		(StarVars),a
;			ld		(T_XNO),hl
			ld		hl,(SAUCERVARS+YNO)
;			add		hl,bc
;			add		hl,bc
			add		hl,bc
			ld		a,l
			ld		(StarVars+1),a
;			ld		(T_YNO),hl

;			call	FindSteps
;			ld		(TempX),hl
;			ld		(TempY),de
*/
			ret

T_Wheel_L:	ld		bc,0x10000 - WHEELXOFF	; left wheel
			jr		SPWheel

T_Wheel_R:	ld		bc,WHEELXOFF		; right wheel

SPWheel:	ld		hl,(SAUCERVARS + XNO)	; link to saucer
			add		hl,bc
			ld		(T_XNO),hl

			ld		a,(XSpeed)			; animate wheel...
			cp		0xff				; full speed?
			jr		nz,TurnWheel

			ld		a,(Loops)			; then used loop counter
			and		1
			jr		TWheel

TurnWheel:	and		16					; else use the x speed

TWheel:		ld		a,G_WHEEL_1 + 0x80	; on screen!
			jr		z,WheelSpin
			inc		a
WheelSpin:	ld		(T_GNO),a

			ld		hl,GROUND - 2		; wobble along the ground
			ld		a,(T_XNO)
			add		a,c
			and		32
			jr		z,Groove
			inc		hl
			inc		hl

Groove:		ld		(T_YNO),hl
			ret

;****************************************
; WORK THE BOMB...
;****************************************

T_BombWait:	ld		a,(T_FLG1)			; bomb jumps up and down...
			inc		a
			ld		(T_FLG1),a
		
			ld		e,a					; up/down vector...
			ld		d,0
			or		a
			jp		p,MaxOut2
			dec		d			
MaxOut2:	ld		hl,(T_YNO)
			add		hl,de

			ld		a,h
			cp		GROUND / 0x100		; hit ground?
			jr		c,NoHitF1

			ld		a,(Rand2)			; random bounce height
			and		3
			cpl
			dec		a
			dec		a
			ld		(T_FLG1),a

			ld		a,(Rand3)			; random x speed
			and		3
			inc		a
			inc		a
			ld		c,a
			ld		hl,(WHEELVARS + SPVARLEN + XNO)
			ld		de,ROVRIGHT
			ld		a,(Loops+1)			; which side of the rover
			and		1
			jr		z,WhichSide			; alternate!
			ld		de,0x10000 - ROVRIGHT - WHEELXOFF - WHEELXOFF
WhichSide:	add		hl,de
			ld		de,(T_XNO)
			sbc		hl,de
			ld		a,h
			or		a
			ld		a,c
			jp		p,RovL
			neg
RovL:		ld		(T_FLG1+1),a		; store x speed low byte only

			ld		hl,GROUND			; place on floor

NoHitF1:	ld		(T_YNO),hl

			ld		a,(T_FLG1+1)		; x speed low byte
			ld		e,a
			ld		d,0
			or		a
			jp		p,NoNg
			dec		d
NoNg:		ld		hl,(T_XNO)			; move in x
			add		hl,de
			ld		(T_XNO),hl

			;*******************

BombCheck:	ld		a,(T_GNO)			; bomb on screen?
			or		a
			ret		p					; don't bother if not!

			ld		hl,(SAUCERVARS+XNO)
			ld		de,(SAUCERVARS+YNO)
			call	CP_16To16
			ret		nc					; no collision

			ld		a,(SAUCERVARS)		; only pick up if you are saucer
			cp		T_SAUCER
			jr		z,PickUpBomb

			ld		a,0x100 - 4			; bounce off saucer	/ rover
			ld		(T_FLG1),a
			ld		hl,GROUND - 16 - 12
			ld		(T_YNO),hl

			ld		a,FXBANG4			; go sound fx
			call	GoFX

			ld		a,(FreezeLR)		; freeze left/right keys
			or		a
			ret		nz
			ld		a,4
			ld		(FreezeLR),a
			ret

PickUpBomb:	ld		a,T_BOMBHELD		; we carry the bomb
			ld		(T_TYP),a
			ld		(Carrying),a

			ld		a,SB_ACCEL			; saucer + bomb acceleration rate
			ld		(Accel),a

			xor		a					; always push upwards!
			ld		(YDir),a

			ld		a,FXLINK1			; go sound fx
			jp		GoFX

;****************************************

T_BombHeld:	ld		hl,(SPData + XNO)	; link to saucer...
			ld		(T_XNO),hl
			ld		hl,(SPData + YNO)
			ld		de,BOMBYOFF
			add		hl,de
			ld		(T_YNO),hl

			ld		a,(OldFUDLR)		; drop the bomb?
			and		BOMBMASK
			ret		nz
			ld		a,(FUDLR)			; drop the bomb?
			and		BOMBMASK
			ret		z

			ld		a,T_BOMBFALL		; change to bomb falling
			ld		(T_TYP),a
			xor		a					; gravity
			ld		(T_FLG1),a
			ld		(Carrying),a		; no longer carrying the bomb

			ld		a,S_ACCEL			; saucer acceleration rate
			ld		(Accel),a

			ld		a,0xf8				; number of bounces - ish
			ld		(T_FLG1+1),a
			ret

;****************************************

T_BombFall:	ld		a,(T_FLG1)			; work gravity
			cp		0x10
			jr		z,MaxOut
			inc		a
			ld		(T_FLG1),a
		
MaxOut:		ld		e,a					; get y vector
			ld		d,0
			or		a
			jp		p,MaxOut1
			dec		d			
MaxOut1:	ld		hl,(T_YNO)
			add		hl,de

			ld		a,h
			cp		GROUND / 0x100		; hit ground?
			ld		a,(T_FLG1+1)		; bounce
			jr		c,NoHitF

			inc		a
			jr		z,BombGoOff
			ld		(T_FLG1+1),a
			ld		(T_FLG1),a

			ld		hl,GROUND			; place on floor
NoHitF:		ld		(T_YNO),hl

			cp		0xf8				; can't pick up before 1st bounce!
			ret		z
			jp		BombCheck			; pick up bomb?


BombGoOff:	ld		a,16				; start background fx
			ld		(SmartDel),a
			call	GoSmart				; fire the smart bomb!
			jp 		GoBigBang

;****************************************
; BADDIES...
;****************************************

T_Baddie:	ld		a,(Loops)			; regulator
			and		31
			ld		a,(T_FLG1)
			jr		nz,NoChan

			inc		a					; change direction
			ld		(T_FLG1),a

NoChan:		add		a,a					; get vectors...
			and		0xe
			add		a,a
			add		a,Num2Vec16 & 0xff
			ld		l,a
			ld		h,Num2Vec16 / 0x100

			ld		e,(hl)
			inc		l
			ld		d,(hl)
			inc		l					; de = x vector
			ld		c,(hl)
			inc		l
			ld		b,(hl)				; bc = y vector

			ld		hl,(T_XNO)			; move in x
			add		hl,de
			ld		(T_XNO),hl

			ld		hl,(T_YNO)			; move in y
			add		hl,bc

			ld		a,h
			cp		GROUND / 0x100		; hit ground?
			jr		c,NoHitG

			ld		a,(T_FLG1)			; bounce
			inc		a					; change direction
			ld		(T_FLG1),a

			ld		hl,GROUND			; place on floor

NoHitG:		ld		(T_YNO),hl

			;*******************

			ld		a,(T_GNO)			; on screen?
			or		a
			ret		p					; ignore if not

			ld		hl,(SAUCERVARS+XNO)
			ld		de,(SAUCERVARS+YNO)
			call	CP_16To16
			ret		nc					; no collision

			ld		de,0x0105			; 100 pts - 1st is num 2nd is index
			call	ScoreAdd			; upscore

			jp 		GoBigBang

;****************************************

T_Bottle:

						

			ret

;****************************************
; COIN BONUS...
;****************************************

T_Coin:		ld		a,(T_GNO)			; on screen?
			or		a
			ret		p					; ignore if not

			ld		hl,(SAUCERVARS+XNO)
			ld		de,(SAUCERVARS+YNO)
			call	CP_16To16
			ret		nc					; no collision

			ld		a,T_COINOFF			; remove coin
			ld		(T_TYP),a

			ld		a,(Rand2)			; gravity thing
			and		7
			add		a,4
			neg
			ld		l,a
			ld		h,0xff
			ld		(T_FLG1),hl

			ld		de,0x0506			; 50 pts - 1st is num 2nd is index
			call	ScoreAdd			; upscore

			ld		a,(Coins)			; coin bonus?
			inc		a
			cp		100
			jr		nz,NoCBon

			ld		a,25*4				; 4 second coin flash
			ld		(CoinDel),a
			ld		de,0x0404			; 4000 pts - 1st is num 2nd is index
			call	ScoreAdd			; upscore
			xor		a

NoCBon:		ld		(Coins),a

			ld		a,FXCOIN			; go sound fx
			jp		GoFX

;****************************************

T_CoinOff:	ld		a,(T_GNO)			; on screen?
			or		a
			jp		p,KillSP			; kill if not

			ld		hl,(T_FLG1)
			inc		hl
			ld		(T_FLG1),hl
			ld		de,(T_YNO)			; y movement
			add		hl,de
			ld		(T_YNO),hl
			ret

;****************************************
; MISSILES...
;****************************************

;
;Multiply 16-bit values (with 16-bit result)
;In: Multiply BC with DE
;Out: HL = result
;
Mult16:		ld		a,b
			ld		b,16
Mult16_Loop:
			add		hl,hl
			sla		c
			rla
			jr		nc,Mult16_NoAdd
			add		hl,de
Mult16_NoAdd:
			djnz	Mult16_Loop
			ret

;
;Multiply 8-bit values
;In:  Multiply H with E
;Out: HL = result
;
Mult8:		ld		d,0
			ld		l,d
			ld		b,8
Mult8_Loop:
			add		hl,hl
			jr		nc,Mult8_NoAdd
			add		hl,de
Mult8_NoAdd:
			djnz	Mult8_Loop
			ret


T_Bullet:	ld		a,(T_FLG1+1)		; timer till death
			dec		a
			ld		(T_FLG1+1),a
			jp		z,GoSmallBang

			ld		hl,(T_XNO)			; move the sprite in the x
			ld		a,(T_FLG1)
			ld		e,a
			ld		d,0
			or		a
			jp		p,BullR
			dec		d
BullR:		add		hl,de
			ld		(T_XNO),hl

			;*******************
/*
	ld		hl,(T_XNO)
;	ld		hl,SAUCERX
	ld		de,(WHEELVARS + XNO)
			or		a
			sbc		hl,de
			ld		a,h
			or		a
			jp		p,NotNeg1
			cpl
			ld		h,a
			ld		a,l
			cpl
			ld		l,a
NotNeg1:	ld		(TempX),hl


		push	hl

	ld		hl,(T_YNO)
;	ld		hl,SAUCERY
	ld		de,(WHEELVARS + YNO)
;	ld		bc,8*12
;	or		a
;	sbc		hl,bc
			or		a
			sbc		hl,de
			ld		a,h
			or		a
			jp		p,NotNeg2
			cpl
			ld		h,a
			ld		a,l
			cpl
			ld		l,a
NotNeg2:	ld		(TempY),hl

		pop		bc
		ex		de,hl



		call	Mult16
		call	Sqr16
		ld		(Temp),a
;		ld		c,a
;		ld		a,0x20
;		sub		c

		cp		0x20
		jr		c,KillBull

			;*******************

KnobOff:
*/

Bull2Bad:	ld		hl,BADDIEVARS

CheckB2B:	ld		a,(hl)				; baddie type
			or		a
			ret		m					; no more to check
			jr		z,NextB2B			; is dead
			cp		T_NOCP				; has no collision
			jr		nc,NextB2B

			push	hl

			inc		hl
			ld		c,(hl)				; bc = x
			inc		hl
			ld		b,(hl)
			inc		hl
			ld		e,(hl)				; de = y
			inc		hl
			ld		d,(hl)
			inc		hl

			ld		a,(hl)				; Gno - bit 7 set if on screen
			or		a
			jp		p,NxtB2B
			
			ld		l,c					; hl = x
			ld		h,b

			call	CP_16To16
			jr		nc,NxtB2B			; no collision

			pop		hl
KillBull:	ld		(hl),T_BIGBNG		; big explosion
			ld		bc,GNO
  			add		hl,bc
   			ld		(hl),G_EXPL_R		; explosion graphic
			inc		hl
			ld		(hl),8				; timer
			ld		a,FXBANG2			; go sound fx
			call	GoFX

			ld		de,0x0105			; 100 pts - 1st is num 2nd is index
			call	ScoreAdd			; upscore
		
			jr		GoSmallBang

NxtB2B:		pop		hl

NextB2B:	ld		de,SPVARLEN
			add		hl,de
			jr		CheckB2B

;****************************************

T_Rocket:	ld		a,(T_FLG1+1)		; timer till death
			dec		a
			ld		(T_FLG1+1),a
			jr		z,GoSmallBang

			srl		a					; y movement
			ld		e,a
			ld		a,9;8
			sub		e
			ld		e,a
			ld		d,0

			ld		hl,(T_YNO)
			sbc		hl,de
			ld		(T_YNO),hl

			jr		Bull2Bad

;****************************************
; TRIGGER STUFF...
;****************************************

GoBigBang:	ld		a,T_BIGBNG			; big explosion
			ld		(T_TYP),a
			ld		a,G_EXPL_R			; explosion graphic
			ld		(T_GNO),a
			ld		a,8					; timer
			ld		(T_FLG1),a
			ld		a,FXBANG1			; go sound fx
			jp		GoFX

GoSmallBang:
			ld		a,T_SMALLBNG		; small explosion
			ld		(T_TYP),a
			ld		a,M_EXPL_R			; explosion graphic
			ld		(T_GNO),a
			ld		a,8					; timer
			ld		(T_FLG1),a
			ld		a,(Rand2)
			and		3
			add		a,FXBANG1			; go sound fx
			jp		GoFX

;****************************************
; FX...
;****************************************

T_BigBang:

T_SmallBang:
			ld		a,(T_GNO)
			xor		1
			ld		(T_GNO),a

			ld		a,(T_FLG1)			; timer
			dec		a
			ld		(T_FLG1),a
			ret		nz

KillSP:		xor		a
			ld		(T_TYP),a			; kill sprite
			ret

;****************************************
; ENGINE FX...
;****************************************

;  a = Graphic Index
; bc = Y Offset

GoEngineFX:	ex		af,af

			ld		a,(Loops)			; pulse release
			and		3
			ret		nz
			push	bc
			call	GetFX				; free FX sprite?
			pop		bc
			ret		nz

			push	hl
			ld		de,(T_XNO)			; setup engine fx
			ld		hl,(T_YNO)			; add y offset...
			add		hl,bc
			ld		c,l
			ld		b,h
			pop		hl
			ld		a,T_ENGINE			; type

			call	SetSP
			ld		a,(Rand2)
			and		3
			add		a,8
			ld		(hl),a				; timer
			xor		a					; flag is done!
			ret

			;*******************

T_Engine:	ld		a,(T_GNO)
			ld		c,a

			ld		a,(T_FLG1)			; timer
			dec		a
			jr		nz,Engines

			ld		a,c					; pop if a bubble
			and		0x7e
			cp		M_BUBBLE
			jp		nz,KillSP			; kill if not

			ld		a,(Rand1)
			and		31
			add		a,20;220
			ld		(UnLinkFX2 + 1),a
			ld		a,FXUNLINK2			; go sound fx
			call	GoFX

			ld		c,M_EXPL_L			; graphic index
			ld		a,8					; timer

Engines:	ld		(T_FLG1),a
			ld		hl,(T_YNO)			; move and animate
			dec		hl
			ld		(T_YNO),hl
			ld		a,c					; animate
			xor		1
			ld		(T_GNO),a
			ret

;****************************************
; FLASHING ARROW FX...
;****************************************

T_Arrow:	ld		a,(T_FLG1)			; delay to stop saucer collision
			or		a
			jr		z,CanColl
			dec		a
			ld		(T_FLG1),a
			jr		NoRoverCP

CanColl:	ld		a,(Carrying)		; no check if carrying an object
			or		a
			jr		nz,NoRoverCP

			ld		hl,GROUND - 0x10	; make sure
			ld		(T_YNO),hl			; arrow always on surface

			ld		hl,(SAUCERVARS+XNO)
			ld		de,(SAUCERVARS+YNO)
			call	CP_16To8
			jr		nc,NoRoverCP		; no collision

			ld		a,T_ROVER			; saucer becomes rover
			ld		(SAUCERVARS + TYP),a

			ld		a,R_ACCEL			; rover acceleration
			ld		(Accel),a

			xor		a					; arrow off
			ld		(T_TYP),a
			ld		(XSpeed),a			; no saucer x/y speed
			ld		(YSpeed),a
			ld		(DrillDel),a		; reset drill delay
			ld		a,T_WHEELL			; wheels follow saucer
			ld		(WHEELVARS),a
			ld		a,T_WHEELR
			ld		(WHEELVARS + SPVARLEN),a

			ld		a,RELEASEDEL		; wheel release delay
			ld		(ReleaseDel),a

			ld		a,FXLINK1			; go sound fx
			jp		GoFX

NoRoverCP:	ld		a,(Loops)
			and		0x10 + 0x4
			ld		hl,GROUND - 0x10
			jr		nz,ArrowBlink
			ld		hl,GROUND + 0x10
ArrowBlink:	ld		(T_YNO),hl
			ret

;****************************************
; COLLISIONS...
;****************************************

; enters with  hl = x1  de = y1 (T_XNO) = x2  (T_YNO) = y2
; returns with carry set if collision!

CP_16To8:	ld		bc,8
			add		hl,bc
			ld		bc,(T_XNO)
			or		a
			sbc		hl,bc
			ld		bc,16
			or		a
			sbc		hl,bc
			ret		nc

			ex		de,hl
			ld		bc,(T_YNO)
			or		a
			sbc		hl,bc
			ld		bc,16
			or		a
			sbc		hl,bc
			ret

;****************************************

CP_8To16:	ld		bc,6
			add		hl,bc
			ld		bc,(T_XNO)
			or		a
			sbc		hl,bc
			ld		bc,12
			or		a
			sbc		hl,bc
			ret		nc

			ex		de,hl
			ld		c,6
			add		hl,bc
			ld		bc,(T_YNO)
			or		a
			sbc		hl,bc
			ld		bc,12
			or		a
			sbc		hl,bc
			ret

;****************************************

CP_16To16:	ld		bc,16
			add		hl,bc
			ld		bc,(T_XNO)
			or		a
			sbc		hl,bc
			ld		bc,16 * 2
			or		a
			sbc		hl,bc
			ret		nc

			ex		de,hl
			ld		c,16
			add		hl,bc
			ld		bc,(T_YNO)
			or		a
			sbc		hl,bc
			ld		bc,16 * 2
			or		a
			sbc		hl,bc
			ret

;****************************************
; ODDS AND ENDS...
;****************************************

; bring on baddies
; loop counter
; print score
; show rover location
; smart bomb display
; display gauges

OddsEnds:

; Temp Bring On A Baddie...

			call	GetBaddie
			jr		nz,NextOdd

			push	hl
			ld		hl,(SAUCERVARS+XNO)
			ld		de,0xb0
			ld		a,(Rand3)
			and		1
			jr		nz,BringOn
			sbc		hl,de
			jr		BringOn1
BringOn:	add		hl,de
BringOn1:	ex		de,hl
			pop		hl

			ld		bc,GROUND - 0x40

			ld		a,(Rand1)
			and		1
			jr		z,Coinz

			ld		a,G_BUBBLE			; graphics index
			ex		af,af
			ld		a,T_BADDIE			; type

			jr		GoSetz

Coinz:		ld		a,G_COIN			; graphics index
			ex		af,af
			ld		a,T_COIN			; type

;			ld		bc,(SAUCERVARS+YNO)

GoSetz:		call	SetSP

			ld		a,(Loops)
			and		7
			ld		(hl),a				; flg1 (vector)

;****************************************
; Loop Counter and Star field...

NextOdd:	ld		hl,(Loops)			; loop counter
			inc		hl
			ld		(Loops),hl

			ld		a,l					; move just one star to another location
			and		STARTOTAL - 1
			add		a,a
			add		a,StarVars & 0xff
			ld		e,a
			ld		d,StarVars / 0x100

			ld		a,(Rand1)
			and		0xfe
			ld		(de),a				; new x
			inc		e
			ld		a,(Rand2)
			and		0x7e
			ld		(de),a				; new y

;****************************************
; Print The Score/Coins/Radar/Tele/Locators etc...

			call	PrScore				; print the score
			call	PrCoins				; print the coin total
			call	LandRadar			; print the landscape radar
			call	MiniTele			; work the mini-montor
			call	Locators			; work the locators

;****************************************
; smart bomb the display...

Smarts:		ld		a,(SmartDel)			
			or		a
			jr		z,PlaceBomb

			dec		a
			ld		(SmartDel),a
			ex		af,af
			call	GoSmart				; fire the smart bomb! - scores too much!
			ex		af,af
			ld		c,a
			and		3
			jr		nz,PlaceBomb

			ld		hl,(SmartFX)		; do bit fx...
			ld		a,l
			cpl
			ld		l,a
			ld		h,a
			inc		c
			dec		c
			jr		nz,PlcBmb

			ld		l,c
			ld		h,c
PlcBmb:		ld		(SmartFX),hl			

;****************************************
; Place Bomb On Map If Ready...

PlaceBomb:	ld		hl,BOMBVARS
			ld		a,(hl)				; bomb already on?
			or		a
			jr		nz,AniBomb

;	ret

			push	hl
			ld		hl,WHEELVARS + XNO
			ld		e,(hl)
			inc		l
			ld		d,(hl)
			ld		bc,GROUND - 16
			ld		hl,WHEELXOFF		; place on rover
			add		hl,de
			ex		de,hl
			pop		hl

			ld		a,G_BOMB			; graphics index
			ex		af,af
			ld		a,T_BOMBWAIT
			call	SetSP
			ld		(hl),0				; flg1 = gravity
			inc		hl
			ld		(hl),0				; flg1+1 = xspd
			ret

;****************************************
; animate the bomb blades...

AniBomb:	ld		a,(Loops)
			add		a,a
			and		6
			add		a,BladeTab & 0xff
			ld		l,a
			ld		h,BladeTab / 0x100
			ld		a,(hl)
			inc		l
			ld		h,(hl)
			ld		l,a

			ld		de,SP_Bomb1
			ldi
			ldi
			ldi
			ld		de,SP_Bomb2
			ldi
			ldi
			ldi
			ld		de,SP_Bomb3
			ldi
			ldi
			ldi
			ld		de,SP_Bomb4
			ldi
			ldi
			ldi
			ret

;****************************************
; CONVERT BLOCK TO SHIFTER BLOCKS...
;****************************************

BlockVert:
			ret
/*
			ld		hl,Block00
			ld		(TempHL),hl

			ld		hl,TestBlock


			ld		e,(hl)
			inc		hl
			ld		d,(hl)
			inc		hl
			ld		c,(hl)
			inc		hl
			ld		b,(hl)
			inc		hl

			ld		a,b
			rra
			rr		e
			rr		d
			rr		c
			rr		b
			rra
			rr		e
			rr		d
			rr		c
			rr		b

			push	hl
			ld		hl,(TempHL)
			ld		(hl),e
			inc		hl
			ld		(hl),d
			inc		hl
;			ld		(TempHL),hl
			ld		de,2 * 16 - 2
			add		hl,de
			ld		(hl),c
			inc		hl
			ld		(hl),b
			pop		hl


			ret

TestBlock:
	defb	164,  0,  0,188,247,  0,  1,127
	defb	255,208, 75,255,255, 88,181,255
	defb	255,178,251,255,255,109,255,255
	defb	255,215,255,255,255,255,255,255
	defb	255,255,255,255,255,255,255,255
	defb	255,248, 63,255,255,240, 95,255
	defb	255,248, 63,255,255,252,127,255
	defb	255,254,255,255,255,255,255,255
*/

;****************************************
; FIND CLOSEST BADDIE...
;****************************************

FindSteps:
			ld		hl,(SAUCERVARS+YNO)
			ld		de,(T_YNO)
			or		a
			sbc		hl,de
			push	hl

			ld		hl,(SAUCERVARS+XNO)
			ld		de,(T_XNO)
			or		a
			sbc		hl,de
			pop		de			
			ret

;****************************************
; FIND SQUARE ROOT OF 16BIT NUMBER...
;****************************************
; In:  hl = value
; Out:  a = result (rounded down)

Sqr16:		ld		de,0x0040
			ld		a,l
			ld		l,h
			ld		h,d
			or		a
			ld		b,8
Sqr16Loop:	sbc		hl,de
			jr		nc,Sqr16Skip
			add		hl,de
Sqr16Skip:	ccf
			rl		d
			add		a,a
			adc		hl,hl
			add		a,a
			adc		hl,hl
			djnz	Sqr16Loop
			ld		a,d
			ret

;****************************************
; INCLUDE FILES...
;****************************************

			include	"source/subs.asm"			; subroutines
			include	"source/menu.asm"			; subroutines
			include "source/library.asm"		; library routines
			include "source/graphics.asm"		; graphics in .asm format
			include "source/bigsprites.asm"		; graphics in .asm format
			include "source/smallsprites.asm"	; graphics in .asm format
			include "source/debug.asm"			; debug routines

ENDCODE:

;****************************************
; END
;****************************************
