;****************************************
;
;	 "SAUCER" - A New Spectrum Game
;
;	Started On The 4th September 2006
;
;
;	(C) 2006 Frobush / Jonathan Smith
;
;****************************************

;****************************************
; SUBROUTINES...
;****************************************

; SetupInts:
; Interrupt:
; GoFX:
; GoFXForce:
; WaitFlyBack:
; Beeper:
; Beep:
; Boop:
; Hiss:
; DumpScreen:
; Landscape:
; LandAttr:
; GoAttrFX:
; WorkAttrFX:
; RKReadPort:
; ReadKeyStick:
; DumpSP:
; SetStars:
; Stars:
; GroundAddr:
; Circle:

;****************************************
; SETUP THE INTERRUPTS...
;****************************************

; disable ints before calling				
; hl = interrupt routine address
; ints enabled on return

SetupInts:	ld      a,OP_JP				; jp opcode
			ld      (INTVECTOR),a
			ld      (INTVECTOR+1),hl

			ld		bc,INTVECTOR / 0x100
			ld		hl,INTTABLE
			ld		a,h
			ld		i,a
			im		2

IntSet:		ld		(hl),c				; setup 257 byte vector table
			inc		hl
			djnz	IntSet
			ld		(hl),c
;			ei
			ret

;****************************************
; WORK THE INTERRUPTS...
;****************************************

Interrupt:	push	af

			ld		a,0x80				; flag the interrupt
			ld		r,a

			ld		a,(SpeedFlag)		; increase timer
			inc		a
			ld		(SpeedFlag),a

			pop		af
			ei
			reti

;****************************************
; START SOUND FX...
;****************************************

GoFX:
;			ex		af,af
;			ld		a,(FXNumber)		; sound already playing - priority?
;			or		a
;			ret		nz
;			ex		af,af

GoFXForce:	ld		(FXNumber),a		; current FX number
			add		a,a
			add		a,FXTable-2 & 0xff
			ld		l,a
			ld		h,FXTable / 0x100
			ld		a,(hl)
			inc		l
			ld		h,(hl)
			ld		l,a
			ld		(FXPointer),hl		; current FX data
			ret

;****************************************
; WAIT FOR FLYBACK...
;****************************************

WaitVBlank:	xor		a
			ld		r,a
			ei

			ld		a,(FXNumber)
			or		a
			jr		nz,Beeper

WaitInt:	ld		a,r
			jp		p,WaitInt
			di
			ret

;****************************************
; IN GAME BEEPER ROUTINES...
;****************************************

Beeper:		ld		hl,(FXPointer)		; table pointer

			ld		a,(hl)				; frequency + code (1 frame info only)
			inc		hl
			cp		LASTFXCODE			; new control code?
			jr		nc,PlayFX0

			add		a,a
			jr		nz,PlayFX
			ld		(FXNumber),a		; no more sound
			jr		WaitInt

PlayFX:		push	hl
			add		a,FXRout & 0xff
			ld		l,a
			ld		h,FXRout / 0x100
			ld		a,(hl)
			inc		l
			ld		h,(hl)
			ld		l,a
			ld		(FXCode),hl			; change current beeper routine
			pop		hl
			ld		a,(hl)

PlayFX0:	ld		(FXPointer),hl		; ready for next time
			ld		e,a					; current frequency
			ld		d,a
			ld		hl,PlayOut			; return address
			push	hl
			ld		hl,(FXCode)
			defb	OP_JPHL

PlayOut:	xor		a
			out		(BEEPPORT),a
			di
			ret

;****************************************
; NORMAL BEEP...
;****************************************

Beep:		ld		a,0x10
			out		(BEEPPORT),a
			ld		b,e
Beep0:		ld		a,r
			ret		m
			djnz	Beep0
			jr		Beep1

Beep1:		ld		a,0
			out		(BEEPPORT),a
			ld		b,d
Beep2:		ld		a,r
			ret		m
			djnz	Beep2
			jr		Beep

;****************************************
; PHASE BEEP...
;****************************************

Boop:		ld		a,0x10
			out		(BEEPPORT),a
			inc		e
			ld		b,e
Boop0:		ld		a,r
			ret		m
			djnz	Boop0
			jr		Boop1
Boop1:		ld		a,0
			out		(BEEPPORT),a
			dec		d
			ld		b,d
Boop2:		ld		a,r
			ret		m
			djnz	Boop2
			jr		Boop

;****************************************
; WHITE NOISE BEEP...
;****************************************

Hiss:		ld		hl,0				; rom pointer

Hisso:		ld		a,0x10
			out		(BEEPPORT),a
			ld		b,(hl)
			inc		hl
Hiss0:		ld		a,r
			ret		m
			djnz	Hiss0
			ld		a,0
			out		(BEEPPORT),a
			ld		b,d
			inc 	d
Hiss2:		ld		a,r
			ret		m
			djnz	Hiss2
			jr		Hisso

;****************************************
; CLEAR THE PLAY AREA...
;****************************************

DumpScreen:	ld		a,WINDOWYL / 2		; play area height
			ex		af,af

			ld		hl,(WindowY)		; is the planet surface visible?
			ld		de,GROUND
			ex		de,hl
			or		a
			sbc		hl,de

			ld		a,h
 			or		a
  			jr		nz,ClearW

			ld		a,l
			sub		WINDOWYL
			jr		nc,ClearW
			sub		WINDOWYL
			srl		a
			jr		z,ClearW
			ex		af,af

			call	ClearW
			ld		(LandHL),hl
			ret

;****************************************
; CLEAR THE WINDOW...
;****************************************

ClearW:		ld		hl,0
			ld		(LandHL),hl
			ex		af,af				; clear the window area...

			ld		hl,SCRNADDR	+ 30	; left side of display
			ld		de,(SmartFX)			; value to fill area
			ld		bc,WindowDl

			ld		(StackStore),sp		; store stack pointer

ClearLoop:	ex		af,af				; clear row of chars

			ld		a,(bc)
			ld		l,a
			inc		c
			ld		a,(bc)
			ld		h,a
			inc		c			

			ld		sp,hl

			push	de					; clear row of pixels
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de

			inc 	h					; down line

			ld		sp,hl

			push	de					; clear row of pixels
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de
			push	de

			ex		af,af
			dec		a
			jr		nz,ClearLoop
			ld		sp,(StackStore)		; restore stack
			ret

;****************************************
; LANDSCAPE...
;****************************************

; LandHL = left of screen addr ('cos of stack print)

Landscape:	ld		hl,(LandHL)			; from stack clear
			ld		a,l
			or		h
			ret		z					; 0 unless needed

			Di
			ld		(StackStore),sp

			ld		a,(WindowX)			; window coord
			ld		c,a
			cpl
			and		8					; + full chr scroll
			srl		a
			srl		a
			srl		a
			ld		e,a

			call	DownLine			; screen at right edge 'cos of stack clear
			ld		a,l
			sub		29					; make it the left edge
			add		a,e					; chr offset if needed
			ld		l,a
			ld		(BodgeHL),hl		; store screen addr

			ld		de,(WindowX)
			ld		a,e
			rr		d
			rra
			rr		d
			rra
			rr		d
			rra
			rr		d
			rra
			ld		e,a
			ld		d,Land / 0x100		; de = map block position

			ld		a,c
			and		6
			add		a,LandShift & 0xff
			ld		l,a
			ld		h,LandShift / 0x100
			ld		c,(hl)
			inc		l
			ld		b,(hl)				; bc = block shift offset

			ld		a,15				; 15 blocks to dump
Scape:		ex		af,af

			ld		a,(de)				; block on left edge
			inc		e					; watch that wrap!
			add		a,LandBlocks & 0xff
			ld		l,a
			ld		h,LandBlocks / 0x100
			ld		a,(hl)
			inc		l
			ld		h,(hl)
			ld		l,a					; hl = block addr
			add		hl,bc				; hl = block + shift
			ld		sp,hl				; sp = new block addr
			exx

			defb	0x21
BodgeHL:	defw	0000				; ld hl,SCRNADDR
			ld		c,l

			ld		b,16 / 2			; maximum height of block

Block:		pop		de
			ld		(hl),e
			inc		l
			ld		(hl),d
			dec	l
			inc		h
			pop		de
			ld		(hl),e
			inc		l
			ld		(hl),d
			dec	l
			inc 	h
			ld 		a,h
			and 	6
			jr	 	nz,Block0
			ld 		a,l
			add 	a,32
			ld 		l,a
			jr	 	c,Block2			; stop drawing the block
			ld		a,h
			sub		8
			ld		h,a

Block0:		pop		de
			ld		(hl),e
			inc		l
			ld		(hl),d
			dec	l
			inc		h
			pop		de
			ld		(hl),e
			inc		l
			ld		(hl),d
			dec	l
			inc 	h
			ld 		a,h
			and 	6
			jr	 	nz,Block1
			ld 		a,l
			add 	a,32
			ld 		l,a
			jr	 	c,Block2			; stop drawing the block
			ld		a,h
			sub		8
			ld		h,a

Block1:		djnz	Block				; try and do a full block

Block2:		ld		a,c					; next screen possition
			inc		a
			inc		a
			ld		(BodgeHL),a
			exx
			ex		af,af
			dec		a
			jr		nz,Scape

			ld		sp,(StackStore)
;			ei
			ret

;****************************************
; Landscape Colours...
;****************************************

LandAttr:	ld		hl,(WindowY)
			ld		bc,GROUND - WINDOWYL + 16 - 31;
			or		a
			ld		a,WINDOWATTR
			sbc		hl,bc				; 0 to 15
			ret		c

			ld		a,l					; get attributes for ground section...
			srl		a
			srl		a
			and		6
			add		a,LandAtTab & 0xff
			ld		l,a
			ld		h,LandAtTab / 0x100

			ld		a,(hl)				; paint the land section...
			inc		l
			ex		af,af
			ld		a,(hl)
			ld		hl,0x20 * 15 + ATTRADDR + 2
			ld		de,0x20 * 15 + ATTRADDR + 3
			ld		(hl),a
			call	LandA0
			ld		hl,0x20 * 14 + ATTRADDR + 2
			ld		de,0x20 * 14 + ATTRADDR + 3
			ex		af,af
			ld		(hl),a

LandA0:		ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ldi
			ret

;****************************************
; ATTRIBUTE FX SCREEN HANDLER...
;****************************************

; de = low res coords
; hl = string pointer

GoAttrFX:	ld		a,(AttrFlag)		; one being displayed?
			or		a
			ret		nz

;			ld		a,FXRKON;FXTEST		; go sound fx
;			call	GoFX

		ld		a,(Rand1)			; demo only...
		and		0x6
		add		a,NibbleFXTab & 0xff
		ld		l,a
		ld		h,NibbleFXTab / 0x100
		ld		a,(hl)
		inc		l
		ld		h,(hl)
		ld		l,a

			ld		e,(hl)				; flash table
			inc		hl
			ld		d,(hl)
			inc		hl
			ld		(AttrFlash),de

			ld		e,(hl)				; low res coords
			inc		hl
			ld		d,(hl)
			inc		hl

			ld		a,8 * 3				; if not then go ahead
			ld		(AttrFlag),a

			ld		(AttrSTR),hl

			ld		l,d					; low res to attr
			ld		h,0
			add		hl,hl
			add		hl,hl
			add		hl,hl
			add		hl,hl
			add		hl,hl
			ld		a,e
			add		a,l
			ld		l,a
			ld		a,h
			add		a,ATTRADDR / 0x100
			ld		h,a					; attribute screen address

			ld		(AttrHL),hl
AttrOut:	ret

;****************************************

WorkAttrFX:	ld		a,(AttrFlag)		; displaying message?
			or		a
			ret		z
			dec		a					; work timer
			ld		(AttrFlag),a
			jr		nz,NotLastAttr

			ld		a,WINDOWATTR
			jr		IsLastAttr

NotLastAttr:
			srl		a
			and		3
			ld		hl,(AttrFlash)
			add		a,l
			ld		l,a
			ld		a,(hl)
IsLastAttr:
			ld		hl,(AttrHL)			; restore the attribute screen...
			ld		bc,(AttrSTR)
			ex		af,af

AttrChar:	ld		a,(bc)				; char from string
			or		a
			jr		z,AttrOut
			push	bc

			sub		65 - 10				; alpha text only
			ld		e,a
			add		a,a
			add		a,a
			add		a,e
			add		a,NibbleChrs & 0xff
			ld		e,a
			ld		d,NibbleChrs / 0x100

			ex		af,af 
			ld		c,a					; colour
			ex		af,af
			ld		b,5					; 5 chrs high

AttrLoop:	ld		a,(de)				; draw the char...
			rlca
			jr		nc,NoAt1
			ld		(hl),c
NoAt1:		inc		l
			rlca
			jr		nc,NoAt2
			ld		(hl),c
NoAt2:		inc		l
			rlca
			jr		nc,NoAt3
			ld		(hl),c
NoAt3:		inc		de

			ld		a,l					; down line
			add		a,32 - 2
			ld		l,a
			adc		a,h
			sub		l
			ld		h,a

			djnz	AttrLoop

			ld		bc,0x10000-160+4	; next position
			add		hl,bc
			pop		bc
			inc		c
			jr		AttrChar

;****************************************
; READ THE KEYBOARD AND RETURN ASCII...
;****************************************

RKReadPort:	ld		hl,RDPortASCII		; quad and ascii table

			ld		d,8					; 8 rows to read

ReadQuad:	ld		c,KEYPORT
			ld		b,(hl)
			in		a,(c)
			ld		c,a

			ld		e,1					; bit mask
			ld		b,5					; 5 possible key presses per row

ReadRow:	ld		a,c					; test if pressing
			and		e
			jr		z,IsPressed
			sla		e					; new mask
			djnz	ReadRow

			ld		a,l					; next row
			add		a,6
			ld		l,a
			adc		a,h
			sub		l
			ld		h,a

			dec		d
			jr		nz,ReadQuad

NotValid:	xor		a
			ret

IsPressed:	ld		a,(Duf)				; stops 1st pass press
			or		a
			ret		m

			ld		a,e					; preserve mask
			ex		af,af

			ld		a,6					; index conversion table
			sub		b
			ld		e,a
			ld		d,0
			ld		b,(hl)				; hi byte of port
			add		hl,de
			ld		c,(hl)				; ascii of key pressed

			ld		hl,AKBuffer			; check if already used
			ld		e,0

CPBuff:		ld		a,(hl)
			or		a
			jr		z,FreeKey
			cp		c
			jr		z,NotValid
			inc		l
			inc		e
			jr		CPBuff

FreeKey:	ld		(hl),c				; store new ascii key in buffer

			ld		a,e					; get index and redef table pointer
			add		a,KeySwap & 0xff
			ld		e,a
			ld		d,KeySwap / 0x100
			ld		a,(de)
			add		a, KeyTable	& 0xff
			ld		e,a

			ld		a,b
			ld		(de),a				; store row in keytable
			inc		e
			ex		af,af
			ld		(de),a				; store the mask

			ld		a,c
			ret

;****************************************
; READ KEYBOARD / JOYSTICKS...
;****************************************

ReadKeyStick:
			ld		a,(FUDLR)			; store last key presses
			ld		(OldFUDLR),a

			ld		a,(MenuSelect)		; get menu selection index
			add		a,Menu2KeyTable & 0xff
			ld		l,a
			ld		h,Menu2KeyTable / 0x100
			ld		a,(hl)				; get table low byte pointer (0 = kempston)
			ld		l,a
			or		a
			jr		z,ReadKempston

ReadKeys:	ld		bc,KEYTOTAL * 0x100	; c = bit store
			ld		d,BitTable / 0x100

ReadLoop:	push	bc

			ld		c,KEYPORT
			ld		b,(hl)
			inc		l
			in		a,(c)
			and		(hl)
			jr		nz,NoPress

			pop		bc
			ld		a,BitTable & 0xff	; if pressed then set that bit
			add		a,b
			ld		e,a
			ld		a,(de)
			or		c
			ld		c,a
			jr		HasPress

NoPress:	pop		bc

HasPress:	inc		l
			djnz	ReadLoop			; read all keys

			ld		a,c
			ld		(FUDLR),a			; store the keyboard bits
			ret

;****************************************

ReadKempston:
			ld		l,a					; l = bit store

			ld		c,KEYPORT
			ld		b,ROWRB				; 
			in		a,(c)
			and		1					; SPACE BAR for bomb (firebutton 2)
			jr		nz,NoSpaceBar

			set		BOMBBIT,l

NoSpaceBar:	ld		b,ROWRMB			; 
			in		a,(c)
			and		1					; ENTER for pause
			jr		nz,NoEnterKey

			set		PAUSEBIT,l

NoEnterKey:	xor		a
			in		a,(JOYPORT)			; read the port - returns bits in the right order
			or		l
			ld		(FUDLR),a
			ret

;****************************************
; DUMP SPRITES...
;****************************************

; could chage some inc hls to inc ls - but leave for now!

DumpSP:		ld		hl,0
			ld		(TempHL2),hl		; offset into bleed graphic
			ld		hl,SPData			; sprite data pointer

DumpSPLoop:	ld		a,(hl)				; type index
			or		a
			jp		z,NoDump			; off?
			ret		m					; 0xff is end of variables

DumpS:		cp		MISSILE				; smaller sprite type?
			jp		c,DumpS0
			call	DumpSPS
			jr		DumpSPLoop

DumpS0:		push	hl
			inc		hl

			ld		a,16/2				; y size / 2
			ex		af,af
			ld		e,(hl)				; 16 bit to 8 - on screen in x?
			inc		hl
			ld		d,(hl)
			inc		hl
			push	hl
			ld		hl,(WindowX)
			ex		de,hl
			or		a
			sbc		hl,de
			ld		a,l
			sub		9					; 18 pixels width - get left edge
			and		0xfe
			ld		e,a
			ld		a,h
			pop		hl
			or		a
			jr		nz,MissDump

			ld		c,(hl)				; 16 bit to 8 bit - on screen in y?
			inc		hl
			ld		b,(hl)
			inc		hl
			push	hl

			ld		l,c
			ld		h,b
			ld		bc,(WindowY)
			sbc		hl,bc				; no carry set!

			ld		a,l
			sub		16					; y coord is bottom 0f sprite - get top
			and		0xfe
			ld		d,a
			ld		a,h
			pop		hl
			or		a
			jr		nz,MissDump
			ld		a,d

			cp		256 - 14			; bleeding off the top?
			jr		c,NoTopBleed

			add		a,16
			ld		d,a
			srl		a
			ex		af,af				; height / 2

			ld		a,16
			sub		d
			ld		d,a
			add		a,a
			add		a,d
			ld		(TempHL2),a			; extra index into graphic

			ld		d,0					; top of screen
			jr		NoMissDump
					
NoTopBleed:	cp		WINDOWYL			; totally off bottom?
			jr		nc,MissDump
			cp		WINDOWYL - 14		; is bleeding off the bottom
			jr		c,NoMissDump

			ld		a,WINDOWYL
			sub		d					; y height
			srl		a
			ex		af,af				; y height / 2

NoMissDump:	ld		a,(hl)				; graphic index
			or		0x80
			ld		(hl),a			; on screen if bit 7 set!
			add		a,a
			add		a,SPGnos & 0xff
			ld		l,a
			ld		h,SPGnos / 0x100
			ld		c,(hl)				; graphic pointer
			inc		l
			ld		b,(hl)
			ld		(TempHL),bc

			ld		a,e					; 2 pixel / 4 images
			and		0x6
			add		a,SP3Offs & 0xff
			ld		l,a
			ld		c,(hl)				; graphic shift offset
			inc		l
			ld		b,(hl)

			ld		hl,(TempHL)
			add		hl,bc				; graphic addr
			ld		bc,(TempHL2)
			add		hl,bc				; bleed pointer
			ld		(TempHL),hl			; store graphic address

			call	Pixad
			call	SP3by3				; or other?
			xor		a
			ld		(TempHL2),a			; clear bleed offset

			pop		hl

NoDump:		ld		de,SPVARLEN
			add		hl,de
			jp		DumpSPLoop


MissDump:

			pop		hl

		ld		de,GNO
		add		hl,de
		ld		a,(hl)
		and		0x7f
		ld		(hl),a
		ld		e,SPVARLEN - GNO
		add		hl,de

;			ld		de,SPVARLEN
;			add		hl,de
			jp		DumpSPLoop

;****************************************
; DUMP MISSILES...
;****************************************

DumpSPS:	push	hl
			inc		hl

			ld		a,8/2				; y size / 2
			ex		af,af

			ld		e,(hl)				; 16 bit to 8 - on screen in x?
			inc		hl
			ld		d,(hl)
			inc		hl
			push	hl
			ld		hl,(WindowX)
			ex		de,hl
			or		a
			sbc		hl,de
			ld		a,l
			sub		5					; 10 pixels width - get left edge
			and		0xfe
			ld		e,a
			ld		a,h
			pop		hl
			or		a
			jr		nz,NotOnScreen

			ld		c,(hl)				; 16 bit to 8 bit - on screen in y?
			inc		hl
			ld		b,(hl)
			inc		hl
			push	hl

			ld		l,c
			ld		h,b
			ld		bc,(WindowY)
			sbc		hl,bc				; no carry set!

			ld		a,l
			sub		8					; y coord is bottom 0f sprite - get top
			and		0xfe
			ld		d,a
			ld		a,h
			pop		hl
			or		a
			jr		nz,NotOnScreen
			ld		a,d

			cp		256 - 6				; bleeding off the top?
			jr		c,NoTopBleedX

			add		a,8
			ld		d,a
			srl		a
			ex		af,af				; height / 2

			ld		a,8
			sub		d
			add		a,a
			ld		(TempHL2),a			; extra index into graphic

			ld		d,0					; top of screen
			jr		NoMissDumpX
					
NoTopBleedX:
			cp		WINDOWYL			; totally off bottom?
			jr		nc,NotOnScreen
			cp		WINDOWYL - 6		; is bleeding off the bottom
			jr		c,NoMissDumpX

			ld		a,WINDOWYL
			sub		d					; y height
			srl		a
			ex		af,af				; y height / 2

NoMissDumpX:
			ld		a,(hl)				; graphic index
			or		0x80
			ld		(hl),a			; on screen if bit 7 set!
			add		a,a
			add		a,SPSGnos & 0xff
			ld		l,a
			ld		h,SPSGnos / 0x100
			ld		c,(hl)				; graphic pointer
			inc		l
			ld		b,(hl)
			ld		(TempHL),bc

			ld		a,e					; 2 pixel / 4 images
			and		0x6
			add		a,SP2Offs & 0xff
			ld		l,a
			ld		c,(hl)				; graphic shift offset
			inc		l
			ld		b,(hl)

			ld		hl,(TempHL)
			add		hl,bc				; graphic addr
			ld		bc,(TempHL2)
			add		hl,bc				; bleed pointer
			ld		(TempHL),hl			; store graphic address

			call	Pixad
			call	SP2by1				; or other?
			xor		a
			ld		(TempHL2),a			; clear bleed offset

			pop		hl
			ld		de,SPVARLEN
			add		hl,de
			ret

NotOnScreen:
			pop		hl

		ld		de,GNO
		add		hl,de
		ld		a,(hl)
		and		0x7f
		ld		(hl),a
		ld		e,SPVARLEN - GNO
		add		hl,de

;			ld		de,SPVARLEN
;			add		hl,de
			ret

;****************************************
; DRAW SPRITE...
;****************************************

; hl = screen address  
;  c = x pos
; 'a = ysize / 2

SP3by3:		ld		(StackStore),sp		; store stack
			ld		sp,(TempHL)

			ex		af,af
			ld		b,a					; y size / 2
SP3Loop:
			pop		de					; print 2 lines...
			ld		a,(hl)
			or		e
			ld		(hl),a
			inc		l
			ld		a,(hl)
			or		d
			ld		(hl),a
			inc		l
			pop		de
			ld		a,(hl)
			or		e
			ld		(hl),a
			dec		l
			dec		l

			inc		h					; down line

			ld		a,(hl)
			or		d
			ld		(hl),a
			inc		l
			pop		de
			ld		a,(hl)
			or		e
			ld		(hl),a
			inc		l
			ld		a,(hl)
			or		d
			ld		(hl),a
			dec		l
			dec		l

			inc		h					; down line
			ld		a,h
			and		7
			jr		nz,SP3Over

			ld 		a,l
			add 	a,0x20
			ld 		l,a
			jr	 	c,SP3Over

			ld 		a,h
			sub 	0x8
			ld 		h,a
SP3Over:
			djnz	SP3Loop

			ld		sp,(StackStore)		; restore stack
			ret

;****************************************
; DRAW MISSILE...
;****************************************

; hl = screen address  
;  c = x pos
; 'a = ysize / 2

SP2by1:		ld		(StackStore),sp		; store stack
			ld		sp,(TempHL)

			ex		af,af
			ld		b,a					; y size / 2

SP2Loop:	pop		de					; print 2 lines...
			ld		a,(hl)
			or		e
			ld		(hl),a
			inc		l
			ld		a,(hl)
			or		d
			ld		(hl),a
			inc		h					; down line
			pop		de
			ld		a,(hl)
			or		d
			ld		(hl),a
			dec		l
			ld		a,(hl)
			or		e
			ld		(hl),a
			inc		h					; down line

			ld		a,h
			and		7
			jr		nz,SP2Over
			ld 		a,l
			add 	a,0x20
			ld 		l,a
			jr	 	c,SP2Over
			ld 		a,h
			sub 	0x8
			ld 		h,a

SP2Over:	djnz	SP2Loop

			ld		sp,(StackStore)		; restore stack
			ret

;****************************************
; STARS...
;****************************************

SetStars:	ld		b,STARTOTAL
			ld		hl,StarVars
Set0:		exx
			call	Random
			exx
			ld		a,(Rand1)			; random x y screen coords - not map!
			and		0xfe
			ld		(hl),a
			inc		l
			ld		a,(Rand2)
			and		0x7e
			ld		(hl),a
			inc		l
			djnz	Set0
			ret

Stars:		ld		hl,StarVars

			ld		a,(WindowX)			; 2 pixel only
			ld		e,a
			ld		a,(WindowY)
			ld		d,a

			ld		c,BitTable / 0x100
			ld		b,STARTOTAL

Star0:		push	de

			ld		a,(hl)				; lock to display x
			sub		e
			ld		e,a
			ex		af,af
			inc		l
			ld		a,(hl)				; lock to display y
			sub		d
			and		0x7e
			ld		d,a
			inc		l
			push	hl

			srl		a
			srl 	a
			srl 	a
			and 	0x18
			or 		0x40

			ld 		h,a
			ld 		a,d
			and 	7
			or 		h
			ld 		h,a

			ld 		a,d
			add		a,a
			add		a,a
			and 	0xe0
			ld 		l,a

			ld 		a,e
			srl 	a
			srl		a
			srl		a
			or		l
			ld		l,a					; hl = screen address.

			ex		af,af
			and		7
			ld		e,a
			ld		d,c					; BitTable / 0x100

			ld		a,(de)				; plot away
			or		(hl)
			ld		(hl),a

NoPlot:		pop		hl
			pop		de
			djnz	Star0
			ret

;****************************************
; GET GROUND ADDRESS AND DATA...
;****************************************

; hl = x coord on ground
; returns with  a = block number  hl = mapaddr

GroundAddr:	ld		a,l
			rr		h
			rra
			rr		h
			rra
			rr		h
			rra
			rr		h
			rra
			dec		a
			dec		a
			ld		l,a
			ld		h,Land / 0x100		; hl = map block position
			ld		a,(hl)
			ret

;****************************************
; CIRCLE ROUTINE...
;****************************************

; Enters with  a = radius (0 to 0xff)
; Returns with  de = x  bc = y

Circle:		ld		c,a
			rlca
			rlca
			and		0x3					; quadrant
			ld		b,a
			add		a,QuadTab & 0xff
			ld		l,a
			ld		h,QuadTab / 0x100
			ld		a,(hl)				; angle
			xor		c
			and		63
			ld		c,a
			add		a,CTab & 0xff
			ld		l,a					; index
			ld		e,(hl)				; x val	lo

			ld		a,CTab + 0x3f & 0xff
			sub		c
			ld		l,a
			ld		c,(hl)				; y val lo

			ld		a,b
			add		a,QuadX & 0xff
			ld		l,a
			ld		a,e
			xor		(hl)
			ld		e,a
			ld		a,b
			add		a,QuadY & 0xff
			ld		l,a
			ld		a,c
			xor		(hl)
			ld		c,a

			ld		d,0					; make results 16 bit
			bit		7,e
			jr		z,PosD
			dec		d

PosD:		ld		b,0
			bit		7,c
			ret		z
			dec		b
			ret

;****************************************
; END
;****************************************
